CC**********************************************************************
C*
C*====================================================================
C* Subroutine AMPFFA(GAL,GAF,GZL,GZF,GCHLA,GCHHA,GCA,GCL,GCH,AMZ,GMZ,
C*             AMHL,AMHL,AMHH,GMHH,AMF,GMF, EIN,EOT,FIN,FOT,AOT, AMP)
C*===============================================================---==
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> f + f-bar A.
C* (Inputs)
C*       GAL(2)   : (R*4) : eeA coupling.
C*       GAF(2)   : (R*4) : ffA coupling.
C*       GZL(2)   : (R*4) : eeZ coupling.
C*       GZF(2)   : (R*4) : ffZ coupling.
C*       GCHLA    : (C*4) : ZhA coupling.
C*       GCHHA    : (C*4) : ZHA coupling.
C*       GCA(2)   : (C*8) : ffA coupling.
C*       GCL(2)   : (C*8) : ffh coupling.
C*       GCH(2)   : (C*8) : ffH coupling.
C*       AMZ      : (R*4) : Z mass.
C*       GMZ      : (R*4) : Z width.
C*       AMHL     : (R*4) : h mass.
C*       GMHL     : (R*4) : h width.
C*       AMHH     : (R*4) : H mass.
C*       GMHH     : (R*4) : H width.
C*       AMF      : (R*4) : f mass.
C*       GMF      : (R*4) : f width.
C*       EIN(6)   : (C*8) : e- spinor.
C*       EOT(6)   : (C*8) : e+ spinor.
C*       FIN(6)   : (C*8) : f-bar spinor.
C*       FOT(6)   : (C*8) : f spinor.
C*       AOT(3)   : (C*8) : scalar.
C* (Outputs)
C*    AMP (0)     : (C*8) : amplitude sum.
C*        (1)     : (C*8) : bremsstrahlung.
C*        (2)     : (C*8) : hA.
C*        (3)     : (C*8) : HA.
C* (Relation)
C*    Invokes AMPFF and subroutines in HELAS.LOAD.
C* (Update Record)
C*    92/04/12  K.Fujii         Original version.
C*    95/03/30  K.Fujii         GHLA and GHHA were made into C*8 and
C*				renamed GCHLA and GCHHA, respectively.
C*    96/07/24  K.Fujii         A bug fixed: GCA --> GCAC in FSO.
C*
CC**********************************************************************
 
      SUBROUTINE AMPFFA(GAL,GAF,GZL,GZF,GCHLA,GCHHA,GCA,GCL,GCH,
     .                  AMZ,GMZ,AMHL,GMHL,AMHH,GMHH,AMF,GMF, 
     .                  EIN,EOT,FIN,FOT,AOT, AMP)
 
      IMPLICIT   REAL*4  ( A-H, O-Z )
      REAL   *4  GAL(2), GAF(2), GZL(2), GZF(2),
     .           AMZ, GMZ, AMHL, GMHL, AMHH, GMHH, AMF, GMF
      COMPLEX*8  GCA(2), GCL(2), GCH(2), GCHLA, GCHHA, 
     .           EIN(6), EOT(6), FIN(6), FOT(6), AOT(3), 
     .           AMP(0:3)
C--
      COMPLEX*8  SPWRK(6), SCWRK(3), TMP(0:2), GCAC(2)
C
C========< Entry Point >================================================
C
C--
C  Bremsstrahlung off f-bar.
C--
      CALL FSIXXX(FIN,AOT,GCA,AMF,GMF,SPWRK)
      CALL AMPFF (GAL,GAF,GZL,GZF,AMZ,GMZ,
     .            EIN,EOT,SPWRK,FOT,TMP)
      AMP(1) = TMP(0)
C--
C  Bremsstrahlung off f.
C--
      GCAC(1) = CONJG(GCA(2))
      GCAC(2) = CONJG(GCA(1))
      CALL FSOXXX(FOT,AOT,GCAC,AMF,GMF,SPWRK)
      CALL AMPFF (GAL,GAF,GZL,GZF,AMZ,GMZ,
     .            EIN,EOT,FIN,SPWRK,TMP)
      AMP(1) = AMP(1) + TMP(0)
C--
C  hA and h to ff-bar.
C--
      CALL HIOXXX(FIN,FOT,GCL,AMHL,GMHL,SCWRK)
      CALL AMPHA(GZL,GCHLA,AMZ,GMZ,SCWRK,AOT,EIN,EOT,AMP(2))
C--
C  HA and H to ff-bar.
C--
      CALL HIOXXX(FIN,FOT,GCH,AMHH,GMHH,SCWRK)
      CALL AMPHA(GZL,GCHHA,AMZ,GMZ,SCWRK,AOT,EIN,EOT,AMP(3))
C--
C  Sum up amplitudes.
C--
      AMP(0) = AMP(1) + AMP(2) + AMP(3)
C--
C  That's it.
C--
      RETURN
      END
