CC**********************************************************************
C*
C*====================================================================
C* Subroutine AMPUDH(GAL,GAU,GAD,GZL,GZU,GZD,GAH,GZH,GC,AMZ,GMZ,AMHM,
C*                   GMHM, AMU,GMU,AMD,GMD, EIN,EOT,FIN,FOT,HOT,AMP)
C*==============================================================---===
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> U + D-bar H-.
C* (Inputs)
C*       GAL(2)   : (R*4) : eeA coupling.
C*       GAU(2)   : (R*4) : ffA coupling.
C*       GZL(2)   : (R*4) : eeZ coupling.
C*       GZU(2)   : (R*4) : ffZ coupling.
C*       GAH      : (R*4) : HHA coupling.
C*       GZH      : (R*4) : HHZ coupling.
C*       GC (2)   : (C*8) : UDH coupling.
C*       AMZ      : (R*4) : Z mass.
C*       GMZ      : (R*4) : Z width.
C*       AMU      : (R*4) : f mass.
C*       GMU      : (R*4) : f width.
C*       AMD      : (R*4) : f mass.
C*       GMD      : (R*4) : f width.
C*       EIN(6)   : (C*8) : e- spinor.
C*       EOT(6)   : (C*8) : e+ spinor.
C*       FIN(6)   : (C*8) : f-bar spinor.
C*       FOT(6)   : (C*8) : f spinor.
C*       HOT(3)   : (C*8) : scalar.
C* (Outputs)
C*    AMP (0)     : (C*8) : amplitude sum.
C*        (1)     : (C*8) : bremsstrahlung.
C*        (2)     : (C*8) : ZH.
C* (Relation)
C*    Invokes AMPFF and subroutines in HELAS.LOAD.
C* (Update Record)
C*    92/04/12  K.Fujii          Original version.
C*    96/07/24  K.Fujii          A bug fixed: GC --> GCC in FSO.
C*
CC**********************************************************************
 
      SUBROUTINE AMPUDH(GAL,GAU,GAD,GZL,GZU,GZD,GAH,GZH,GC,AMZ,GMZ,AMHM,
     .                  GMHM, AMU,GMU,AMD,GMD, EIN,EOT,FIN,FOT,HOT,AMP)
 
      IMPLICIT   REAL*4  ( A-H, O-Z )
      REAL   *4  GAL(2), GAU(2), GAD(2), GZL(2), GZU(2), GZD(2),
     .           GAH, GZH, AMZ, GMZ, AMHM, GMHM, AMU, GMU, AMD, GMD
      COMPLEX*8  GC(2), EIN(6), EOT(6), FIN(6), FOT(6),
     .           HOT(3), AMP(0:2)
C--
      COMPLEX*8  SPWRK(6), SCWRK(3), TMP(0:2), GCC(2)
C
C========< Entry Point >================================================
C
C--
C  Bremsstrahlung off D-bar.
C--
      CALL FSIXXX(FIN,HOT,GC,AMU,GMU,SPWRK)
      CALL AMPFF (GAL,GAU,GZL,GZU,AMZ,GMZ,
     .            EIN,EOT,SPWRK,FOT,TMP)
      AMP(1) = TMP(0)
C--
C  Bremsstrahlung off U.
C--
      GCC(1) = CONJG(GC(2))
      GCC(2) = CONJG(GC(1))
      CALL FSOXXX(FOT,HOT,GCC,AMD,GMD,SPWRK)
      CALL AMPFF (GAL,GAD,GZL,GZD,AMZ,GMZ,
     .            EIN,EOT,FIN,SPWRK,TMP)
      AMP(1) = AMP(1) + TMP(0)
C--
C  H+H- and H+ to U D-bar.
C--
      CALL HIOXXX(FIN,FOT,GC,AMHM,GMHM,SCWRK)
      CALL AMPHHC(GAL,GZL,GAH,GZH,AMZ,GMZ,SCWRK,HOT,EIN,EOT,TMP)
      AMP(2) = TMP(0)
C--
C  Sum up amplitudes.
C--
      AMP(0) = AMP(1) + AMP(2)
C--
C  That's it.
C--
      RETURN
      END
