CC**********************************************************************
C*
C*====================================----===
C* Subroutine FULFFH(ITYP,IDP,IHEL,PV,AMP2)
C*====================================----===
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> f + f-bar H.
C* (Inputs)
C*       ITYP     : (I*4) : (1,2,3,4)=(ffh,ffH,ffA,duH-).
C*       IDP(1,i) : (I*4) : generation number.
C*          (2,i) : (I*4) : T3 + 1.5.
C*          (3,i) : (I*4) : l/q flag; (1,2) = (l,q).
C*       IHEL( i) : (I*4) : helicity combination.
C*       PV(*, i) : (R*4) : 4-momentum of i-th particle.
C*                        : i =  1 for  e-.
C*                        :   =  2 for  e+.
C*                        :   =  3 for  f-bar.
C*                        :   =  4 for  f.
C*                        :   =  5 for  H.
C* (Outputs)
C*    AMP2(0)     : (R*4) : (amplitude sum)**2.
C*        (1)     : (R*4) : H bremsstrahlung off ffbar.
C*        (2)     : (R*4) : ZH and Z to ffbar.
C* (Relation)
C*    Invokes AMPFFH and subroutines in HELAS.LOAD.
C* (Update Record)
C*    92/04/12  K.Fujii         Original version.
C*    95/03/27  K.Fujii        	Modified to use INSMCN and INSSCN.	
C*    95/03/30  K.Fujii        	Added ITYP to dummy arguments.
C*    95/03/30  K.Fujii         GZHA was made into C*8 and renamed 
C*				GCZHA.
C*
CC**********************************************************************
 
      SUBROUTINE FULFFH(ITYP,IDP,IHEL,PV,AMP2)
 
      IMPLICIT   REAL*4  ( A-H, O-Z )
=EXPAND 'gen_lib/hg/HGCONS.inc'
=EXPAND 'gen_lib/sm/SMCONS.inc'
=EXPAND 'gen_lib/sm/SMPTAB.inc'
=EXPAND 'gen_lib/sm/SMCUPL.inc'
      INTEGER*4  ITYP, IDP(3,*), IHEL(*)
      REAL   *4  PV(0:3,*), AMP2(0:3)
C--
      REAL   *4  GAF(2), GZF(2)
      COMPLEX*8  GCA(2), GCH(2), GCZHA
      COMPLEX*8  EIN(6), EOT(6), FIN(6), FOT(6), HOT(3),  AMP(0:3)
      DATA NCALL / 0 /
C
C========< Entry Point >================================================
C
C--
C  Prepare coupling constants.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         CS2W   = xCOS2W
         AMSE   = AMF(1,2,1)
         AMSW   = AMW
         AMSZ   = AMZ
         GAMZ   = GMZTOT
C--
         IF ( IDP(3,3).EQ.1 ) THEN
            IF ( IDP(2,3).EQ.1 ) THEN
               GAF(1) = 0
               GAF(2) = 0
               GZF(1) = GZN(1)
               GZF(2) = GZN(2)
            ELSE
               GAF(1) = GAL(1)
               GAF(2) = GAL(2)
               GZF(1) = GZN(1)
               GZF(2) = GZN(2)
            ENDIF
         ELSE
            IF ( IDP(2,3).EQ.1 ) THEN
               GAF(1) = GAU(1)
               GAF(2) = GAU(2)
               GZF(1) = GZU(1)
               GZF(2) = GZU(2)
            ELSE
               GAF(1) = GAD(1)
               GAF(2) = GAD(2)
               GZF(1) = GZD(1)
               GZF(2) = GZD(2)
            ENDIF
         ENDIF
      ENDIF
C--
C  Prepare coupling constants involving Higgs.
C--
      AMSA   = AMA
      GAMA   = GMA
      AMSF   = AMF(IDP(1,3),IDP(2,3),IDP(3,3))
      GAMF   = GMF(IDP(1,3),IDP(2,3),IDP(3,3))
C--
      CSAMB  = CSALF*CSBET + SNALF*SNBET
      SNAMB  = SNALF*CSBET - CSALF*SNBET
C--
      IF ( ITYP.EQ.1 ) THEN
         GZZH   = -SNAMB*GW*AMSW/CS2W/2
         GCZHA  = (0.,1.)*CSAMB*GW/SQRT(CS2W)/2
C--
         IF ( IDP(2,3).EQ.1 ) THEN
            FCGCH  = -CSALF/SNBET
            FCGCA  = 1/TNBET
         ELSE
            FCGCH  = SNALF/CSBET
            FCGCA  = TNBET
         ENDIF
         GCH(1) =  FCGCH*GW*AMSF/AMSW/2
         GCH(2) =  GCH(1)
         GCA(1) = -(0.,1.)*FCGCA*GW*AMSF/AMSW/2
         GCA(2) = -GCA(1)
      ELSE IF ( ITYP.EQ.2 ) THEN
         GZZH   = +CSAMB*GW*AMSW/CS2W/2
         GCZHA  = (0.,1.)*SNAMB*GW/SQRT(CS2W)/2
C--
         IF ( IDP(2,3).EQ.1 ) THEN
            FCGCH  = -SNALF/SNBET
            FCGCA  = 1/TNBET
         ELSE
            FCGCH  = -CSALF/CSBET
            FCGCA  = TNBET
         ENDIF
         GCH(1) =  FCGCH*GW*AMSF/AMSW/2
         GCH(2) =  GCH(1)
         GCA(1) = -(0.,1.)*FCGCA*GW*AMSF/AMSW/2
         GCA(2) = -GCA(1)
      ENDIF
C--
C  Prepare spinors for external lines.
C--
      CALL IXXXXX(PV(0,1),AMSE,IHEL(1),+1,EIN)
      CALL OXXXXX(PV(0,2),AMSE,IHEL(2),-1,EOT)
      CALL IXXXXX(PV(0,3),AMSF,IHEL(3),-1,FIN)
      CALL OXXXXX(PV(0,4),AMSF,IHEL(4),+1,FOT)
C--
C  Prepare wave function for H.
C--
      CALL SXXXXX(PV(0,5),+1,HOT)
C--
C  Calculate amplitude**2.
C--
      CALL AMPFFH(GAL,GAF,GZL,GZF,GZZH,GCZHA,GCH,GCA,AMSZ,GAMZ,
     .            AMSA,GAMA,AMSF,GAMF, EIN,EOT,FIN,FOT,HOT,AMP)
      FACT    = 3**(IDP(3,3)-1)
      AMP2(0) = FACT*ABS(AMP(0))**2
      AMP2(1) = FACT*ABS(AMP(1))**2
      AMP2(2) = FACT*ABS(AMP(2))**2
      AMP2(3) = FACT*ABS(AMP(3))**2
C--
C  That's it.
C--
      RETURN
      END
