CC**********************************************************************
C*
C*===============================----===
C* Subroutine FULUDH(IDP,IHEL,PV,AMP2)
C*===============================----===
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> U + D-bar H-.
C* (Inputs)
C*       IDP(1,i) : (I*4) : generation number.
C*          (2,i) : (I*4) : T3 + 1.5.
C*          (3,i) : (I*4) : l/q flag; (1,2) = (l,q).
C*       IHEL( i) : (I*4) : helicity combination.
C*       PV(*, i) : (R*4) : 4-momentum of i-th particle.
C*                        : i =  1 for  e-.
C*                        :   =  2 for  e+.
C*                        :   =  3 for  D-bar.
C*                        :   =  4 for  U.
C*                        :   =  5 for  H-.
C* (Outputs)
C*    AMP2(0)     : (R*4) : (amplitude sum)**2.
C*        (1)     : (R*4) : H- bremsstrahlung off U D-bar.
C*        (2)     : (R*4) : H+H- and H+ to U D-bar.
C* (Relation)
C*    Invokes AMPFFH and subroutines in HELAS.LOAD.
C* (Update Record)
C*    92/04/12  K.Fujii         Original version.
C*    95/03/27  K.Fujii        	Modified to use INSMCN and INSSCN.	
C*    96/07/23  K.Fujii        	Take into account V_KM.	
C*
CC**********************************************************************
 
      SUBROUTINE FULUDH(IDP,IHEL,PV,AMP2)
 
      IMPLICIT   REAL*4  ( A-H, O-Z )
=EXPAND 'gen_lib/hg/HGCONS.inc'
=EXPAND 'gen_lib/sm/SMCONS.inc'
=EXPAND 'gen_lib/sm/SMPTAB.inc'
=EXPAND 'gen_lib/sm/SMCUPL.inc'
      INTEGER*4  IDP(3,*), IHEL(*)
      REAL   *4  PV(0:3,*), AMP2(0:3)
C--
      REAL   *4  GAH, GZH
      COMPLEX*8  GC(2)
      COMPLEX*8  EIN(6), EOT(6), FIN(6), FOT(6), HOT(3),  AMP(0:2)
      DATA NCALL / 0 /
C
C========< Entry Point >================================================
C
C--
C  Prepare coupling constants.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         AMSE   = AMF(1,2,1)
         AMSW   = AMW
         AMSZ   = AMZ
         GAMZ   = GMZTOT
      ENDIF
C--
C  Prepare coupling constants involving Higgs.
C--
      AMSHM  = AMHM
      GAMHM  = GMHM
      AMSD   = AMF(IDP(1,3),IDP(2,3),IDP(3,3))
      GAMD   = GMF(IDP(1,3),IDP(2,3),IDP(3,3))
      AMSU   = AMF(IDP(1,4),IDP(2,4),IDP(3,4))
      GAMU   = GMF(IDP(1,4),IDP(2,4),IDP(3,4))
      GAH    = GAL(1)
      GZH    = GZL(1)
      GC(1)  = GW*AMSU/AMSW/SQRT(2.)/TNBET
      GC(2)  = GW*AMSD/AMSW/SQRT(2.)*TNBET
C--
C  Prepare spinors for external lines.
C--
      CALL IXXXXX(PV(0,1),AMSE,IHEL(1),+1,EIN)
      CALL OXXXXX(PV(0,2),AMSE,IHEL(2),-1,EOT)
      CALL IXXXXX(PV(0,3),AMSF,IHEL(3),-1,FIN)
      CALL OXXXXX(PV(0,4),AMSF,IHEL(4),+1,FOT)
C--
C  Prepare wave function for H.
C--
      CALL SXXXXX(PV(0,5),+1,HOT)
C--
C  Calculate amplitude**2.
C--
      CALL AMPUDH(GAL,GAU,GAD,GZL,GZU,GZD,GAH,GZH,GC,AMSZ,GAMZ,AMSHM,
     .            GAMHM, AMSU,GAMU,AMSD,GAMD, EIN,EOT,FIN,FOT,HOT,AMP)
      FACT    = 3**(IDP(3,3)-1)*VKM(IDP(1,4),IDP(1,3),IDP(3,3))**2
      AMP2(0) = FACT*ABS(AMP(0))**2
      AMP2(1) = FACT*ABS(AMP(1))**2
      AMP2(2) = 0
      AMP2(3) = FACT*ABS(AMP(2))**2
C--
C  That's it.
C--
      RETURN
      END
