CC**********************************************************************
C*
C*============================================------===
C* Subroutine SETPV4(RS,CSZ,FIZ,AMTT2,CST,FIT,WAT,PV)
C*============================================------===
C*
C* (Purpose)
C*    Set 4-momenta for e+ + e- ---> t + t-bar + Z.
C* (Inputs)
C*       RS      : (R*4) : sqrt(s).
C*       CSZ     : (R*4) : cos(theta_Z).
C*       FIZ     : (R*4) : phi_Z.
C*       AMTT2   : (R*4) : invariant mass squared for t t-bar system.
C*       CST     : (R*4) : cos(theta_t) in t t-bar rest frame.
C*       FIT     : (R*4) : phi_t in t t-bar rest frame.
C* (Output)
C*       WAT     : (R*4) : phase space weight.
C*       PV(*,1) : (R*4) : 4-momentum of e-.
C*         (*,2) : (R*4) : 4-momentum of e+.
C*         (*,3) : (R*4) : 4-momentum of t.
C*         (*,4) : (R*4) : 4-momentum of t-bar.
C*         (*,5) : (R*4) : 4-momentum of Z.
C* (Relation)
C*    Invokes UVZERO, UHSETF, UBTRAN, TTATTZ, and library routines
C*    in 'T#FP.HELAS.FORT'.
C* (Update Record)
C*    91/06/10  K.Fujii          Original version.
C*
CC**********************************************************************
 
      SUBROUTINE SETPV4(RS,CSZ,FIZ,AMTT2,CST,FIT,WAT,PV)
 
      IMPLICIT REAL*4  ( A-H, O-Z    )
=EXPAND 'TKSF.G#XCXC.FORT/SMCONS.inc'
=EXPAND 'TKSF.G#XCXC.FORT/SMPTAB.inc'
      PARAMETER        ( IxPRC = 1 )
      REAL*4    RS, CSZ, FIZ, AMTT2, CST, FIT, WAT, PV(0:3,5)
      REAL*4    QV(0:3,2), EB(3,3)
      DATA NCALL /0/
C--
C  Statement function.
C--
      BETA(X1,X2) = SQRT( 1 - 2*(X1+X2) + (X1-X2)**2 )
C
C========< Entry Point >================================================
C
C--
C  Constants.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         AME    = AMF(1,2,1)
         AMZ0   = AMZ
         GMZ0   = GMZTOT
         AMZ2   = AMZ*AMZ
         FACT   = xGV2PB/(64*x2PI**5)
      ENDIF
C--
      AMT    = AMF(3,1,2)
      GMT    = GMF(3,1,2)
      AMT2   = AMT*AMT
      AMH0   = AMH
      GMH0   = GMHTOT
      AMH2   = AMH*AMH
C--
C  Set 4-momenta.
C     PV(*,1) = 4-momentum of e-
C       (*,2) = 4-momentum of e+
C       (*,3) = 4-momentum of t-bar
C       (*,4) = 4-momentum of t
C       (*,5) = 4-momentum of Z
C--
      CALL UVZERO(20*IxPRC,PV)
      CALL UVZERO( 8*IxPRC,QV)
C--
      EBM     = RS/2
      S       = RS*RS
      PBM     = SQRT((EBM-AME)*(EBM+AME))
      BT0     = PBM/EBM
      PV(0,1) = EBM
      PV(3,1) = PBM
      CALL PMIRRx(PV(0,1),PV(0,2))
C--
      BT1     = BETA(AMH2/S,AMTT2/S)
      APZ     = EBM*BT1
      SNZ     = SQRT((1-CSZ)*(1+CSZ))
      PV(0,5) = SQRT(APZ*APZ+AMH2)
      PV(1,5) = APZ*SNZ*COS(FIZ)
      PV(2,5) = APZ*SNZ*SIN(FIZ)
      PV(3,5) = APZ*CSZ
C--
      AMTT    = SQRT(AMTT2)
      BT2     = BETA(AMT2/AMTT2,AMT2/AMTT2)
      APT     = (AMTT/2)*BT2
      SNT     = SQRT((1-CST)*(1+CST))
      PV(0,4) = SQRT(APT*APT+AMT2)
      PV(1,4) = APT*SNT*COS(FIT)
      PV(2,4) = APT*SNT*SIN(FIT)
      PV(3,4) = APT*CST
C--
      CALL PMIRRx(PV(0,4),PV(0,3))
C--
      QV(0,1) = RS
      CALL PSUBxx(QV(0,1),PV(0,5),QV(0,2))
      CALL UHSETF(QV(1,2),EB)
      CALL UBTRAN(PV(1,3),EB,PV(1,3))
      CALL UBTRAN(PV(1,4),EB,PV(1,4))
      CALL BOOSTx(PV(0,3),QV(0,2),PV(0,3))
      CALL BOOSTx(PV(0,4),QV(0,2),PV(0,4))
C--
C  Calculate phase space weight.
C--
      WAT = FACT*BT1*BT2/(2*S*BT0)
C--
C  That's it.
C--
      RETURN
      END
