CC**********************************************************************
C*
C*==================================-----===
C* Subroutine SGFFH(ITYP,IDP,IHEL,X,SG,PV)
C*==================================-----===
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> f + f-bar + H.
C* (Inputs)                            _   _       _
C*       ITYP     : (I*4) : (1,2,3,4)=(ffh,ffH,ffA,duH-).
C*       IDP(1,i) : (I*4) : generation number.
C*          (2,i) : (I*4) : T3 + 1.5.
C*          (3,i) : (I*4) : l/q flag; (1,2) = (l,q).
C*       IHEL( i) : (I*4) : helicity combination.
C*                        : i = (1,2,3,4,5,6)
C*                        :   = (e-,e+,f-bar,f,H)
C*       X(1,1)   : (R*4) : s.
C*        (2,1)   : (R*4) : cos(theta_H).
C*        (3,1)   : (R*4) : phi_H.
C*       X(1,2)   : (R*4) : invariant mass squared for f-bar-f.
C*        (2,2)   : (R*4) : cos(theta_f_bar) in f-bar-f rest frame.
C*        (3,2)   : (R*4) : phi_f_bar in f-bar-f rest frame.
C* (Output)
C*       PV(*,i)  : (R*4) : 4-momentum of i-th particle, where
C*                        : nubering convention is that of IDP.
C*       SG       : (R*4) : d(sigma)/dX1dX2....dXn.
C* (Relation)
C*    Invokes UVZERO, UHSETF, UBTRAN, FULFFH, and library routines
C*    in 'T#FP.HELAS.FORT'.
C* (Update Record)
C*    90/07/03  K.Fujii         Original version.
C*    92/04/04  K.Fujii         No W decays.
C*    95/03/27  K.Fujii        	Modified to use INSMCN and INSSCN.	
C*    95/03/30  K.Fujii        	Added ITYP to dummy arguments.
C*
CC**********************************************************************
 
      SUBROUTINE SGFFH(ITYP,IDP,IHEL,X,SG,PV)
 
      IMPLICIT     REAL*4  ( A-H, O-Z    )
=EXPAND 'gen_lib/hg/HGCONS.inc'
=EXPAND 'gen_lib/sm/SMCONS.inc'
=EXPAND 'gen_lib/sm/SMPTAB.inc'
      PARAMETER    ( IxPRC = 1 )
      INTEGER*4    ITYP, IDP(3,*), IHEL(*)
      REAL   *4    X(3,*), SG, PV(0:3,*)
      REAL   *4    QV(0:3,2), EC(3,3), EB(3,3), EA(3,3), AMP2(0:3)
      DATA NCALL /0/
C
C========< Entry Point >================================================
C
C--
C  Constants.
C  Spin average is taken only for positron.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         SPIN   = 2
         NB     = 2
         FACT   = xGV2PB*x2PI/(x4PI**(3*NB))/SPIN
         AME    = AMF(1,2,1)
         CALL UVZERO(9,EC)
         EC(1,1) = 1
         EC(2,2) = 1
         EC(3,3) = 1
C--
         IF ( ITYP.EQ.1 ) THEN
            AMSHG = AMHL
         ELSE IF ( ITYP.EQ.2 ) THEN
            AMSHG = AMHH
         ELSE IF ( ITYP.EQ.3 ) THEN
            AMSHG = AMA
         ELSE IF ( ITYP.EQ.4 ) THEN
            AMSHG = AMHM
         ENDIF
      ENDIF
C--
C  Set 4-momenta.
C--
      CALL UVZERO(20*IxPRC,PV)
      CALL UVZERO( 8*IxPRC,QV)
C
C-- e+ + e- --> CM.
C
      S       = X(1,1)
      RS      = SQRT(S)
      EBM     = RS/2
      PBM     = SQRT((EBM-AME)*(EBM+AME))
      BT0     = PBM/EBM
      PV(0,1) = EBM
      PV(3,1) = PBM
      CALL PMIRRx(PV(0,1),PV(0,2))
      QV(0,1) = RS
C
C-- CM --> H f-bar-f
C
      AM12 = AMSHG**2
      AM22 = X(1,2)
      CALL UHPHS2(1,QV(0,1),AM12,AM22,EC,X(2,1),X(3,1),
     .            PV(0,5),QV(0,2),BT1,EA)
      IF ( BT1.EQ.0. )                           GO TO 9999
C
C-- f-bar-f --> f-bar + f
C
      AM12 = AMF(IDP(1,3),IDP(2,3),IDP(3,3))**2
      AM22 = AMF(IDP(1,4),IDP(2,4),IDP(3,4))**2
      CALL UHPHS2(2,QV(0,2),AM12,AM22,EC,X(2,2),X(3,2),
     .            PV(0,3),PV(0,4),BT2,EB)
      IF ( BT2.EQ.0. )                           GO TO 9999
C--
C  Calculate phase space weight.
C--
      WAT = FACT*BT1*BT2/(2*S*BT0)
C--
C  Calculate amplitude squared.
C--
      IF ( ITYP.EQ.1 .OR. ITYP.EQ.2 ) THEN
         CALL FULFFH(ITYP,IDP,IHEL,PV,AMP2)
      ELSE IF ( ITYP.EQ.3 ) THEN
         CALL FULFFA(IDP,IHEL,PV,AMP2)
      ELSE IF ( ITYP.EQ.4 ) THEN
         CALL FULUDH(IDP,IHEL,PV,AMP2)
      ELSE
         PRINT *, '>>>> SGFFH does not support ITYP = ', ITYP
         PRINT *, ' Forced STOP. '
         STOP
      ENDIF
C--
C  Differenctial cross section.
C--
      SG  = AMP2(0)*WAT
CBRONLY       SG  = AMP2(1)*WAT
CZHONLY       SG  = AMP2(2)*WAT
CHHONLY       SG  = AMP2(3)*WAT
C--
C  That's it.
C--
      RETURN
C--
C  Kinematically forbidden.
C--
9999  SG = 0
      RETURN
      END
