CC**********************************************************************
C*
C*===============================----===
C* Subroutine FULNNH(IDP,IHEL,PV,AMP2)
C*===============================----===
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> nu nubar h.
C* (Inputs)
C*       IDP(1,i) : (I*4) : generation number.
C*          (2,i) : (I*4) : T3 + 1.5.
C*          (3,i) : (I*4) : l/q flag; (1,2) = (l,q).
C*       IHEL( i) : (I*4) : helicity combination.
C*       PV(*, i) : (R*8) : 4-momentum of i-th particle.
C*                        : i =  1 for  e-.
C*                        :   =  2 for  e+.
C*                        :   =  3 for  nu.
C*                        :   =  4 for  nubar.
C*                        :   =  5 for  h.
C* (Outputs)
C*    AMP2(0)     : (R*8) : (amplitude sum)**2.
C*        (1)     : (R*8) : Higgs.
C* (Relation)
C*    Invokes AMPNNH and subroutines in HELAS.LOAD.
C* (Update Record)
C*    97/02/09  K.Fujii          Original version.
C*
CC**********************************************************************
 
      SUBROUTINE FULNNH(IDP,IHEL,PV,AMP2)
 
      IMPLICIT   REAL*8  ( A-H, O-Z )
=EXPAND 'dkin_lib/SMCONS.inc'
=EXPAND 'dkin_lib/SMPTAB.inc'
=EXPAND 'dkin_lib/SMCUPL.inc'
      PARAMETER  ( NP = 5 )
      INTEGER*4  IDP(3,NP), IHEL(NP)
      REAL   *8  PV(0:3,NP), AMP2(0:1)
C--
      COMPLEX*16 SPINOR(6,NP), SCH(3), AMP(0:1)
      INTEGER*4  NF(NP,2)
      DATA  NF  / +1, -1,   +1, -1,  +1,
     .            +1, -1,   -1, +1,  -1 /
      DATA NCALL / 0 /
C
C========< Entry Point >================================================
C
C--
C  Prepare coupling constants.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL     = 1
         AMSE      = AMF(1,2,1)
         AMSW      = AMW
         GAMW      = GMWTOT
         AMSH      = AMH
         GAMH      = GMHTOT
C--
         GWWH      = GW*AMSW
      ENDIF
C--
C  Prepare spinors for external lines.
C--
      DO 100 IP = 1, NP-1
         IF ( NF(IP,2).GT.0 ) THEN
            CALL IXXXXX(PV(0,IP),AMF(IDP(1,IP),IDP(2,IP),IDP(3,IP)),
     .                  IHEL(IP),NF(IP,1), SPINOR(1,IP))
         ELSE
            CALL OXXXXX(PV(0,IP),AMF(IDP(1,IP),IDP(2,IP),IDP(3,IP)),
     .                  IHEL(IP),NF(IP,1), SPINOR(1,IP))
         ENDIF
100   CONTINUE
C--
C  Prepare wave function for Higgs.
C--
      CALL SXXXXX( PV(0,NP), NF(NP,1), SCH )
C--
C  Calculate amplitude**2.
C--
      CALL AMPNNH(GWF, GWWH, AMSW, GAMW, AMSE, AMSH, GAMH,
     .                          SPINOR(1, 1),SPINOR(1, 2),
     .                          SPINOR(1, 3),SPINOR(1, 4),
     .                                                SCH, AMP)
C--
      AMP2(0) = ABS(AMP(0))**2
      AMP2(1) = ABS(AMP(1))**2
C--
C  That's it.
C--
      RETURN
      END
