 
CC**********************************************************************
C*
C*==============================---------==
C* Subroutine ELBXT2(EP,AMP,AMD,EMIN,EMAX)
C*==============================---------==
C*
C* (Purpose)
C*    Calculate the minimum and maximum of the energy of the
C*    massless daughter 1 from the 2-body decay:
C*        P --> D + 1
C*    where the parent( P ) and the dauter( D ) have fixed masses.
C* (Inputs)
C*    EP    :(R*8): the parent E in the lab. frame.
C*    AMP   :(R*8): the parent mass.
C*    AMD   :(R*8): the fixed daughter mass.
C* (Outputs)
C*    EMIN  :(R*8): the minimum E of the 1 in the lab. frame.
C*    EMAX  :(R*8): the maximum E of the 1 in the lab. frame.
C* (Update Record)
C*    92/06/26  K.Fujii                Original version.
C*
CC**********************************************************************
 
      SUBROUTINE ELBXT2(EP,AMP,AMD,EMIN,EMAX)
 
      IMPLICIT   REAL*8 ( A-H, O-Z )
      REAL*8     EP, AMP, AMD, EMIN, EMAX
C
C==================< Entry Point >======================================
C
C--
C  Calculate beta and gamma.
C--
      GM   = EP/AMP
      BT   = SQRT( (GM-1)*(GM+1) )/GM
      X    = AMD/AMP
      PMX  = (AMP/2)*(1-X)*(1+X)
      EMX  = SQRT( PMX*PMX + AMD*AMD )
C--
C  Calculate Emin.
C--
      EMIN = EP - GM*( EMX + BT*PMX )
C--
C  Calculate Emax.
C--
      EMAX = EP - GM*( EMX - BT*PMX )
C--
C  That's it.
C--
      RETURN
      END
