 
CC**********************************************************************
C*
C*==============================---------==
C* Subroutine ELBXTR(EP,AMP,AMD,EMIN,EMAX)
C*==============================---------==
C*
C* (Purpose)
C*    Calculate the minimum and maximum of the energy of the
C*    2-body system from the 3-body decay:
C*        P --> D + 1 + 2
C*    where the parent( P ) and the dauter( D ) have fixed masses
C*    while the 2-body system consisting of the massless daughers
C*    ( 1 and 2 ) does not.
C* (Inputs)
C*    EP    :(R*8): the parent E in the lab. frame.
C*    AMP   :(R*8): the parent mass.
C*    AMD   :(R*8): the fixed daughter mass.
C* (Outputs)
C*    EMIN  :(R*8): the minimum E of the (1+2) in the lab. frame.
C*    EMAX  :(R*8): the maximum E of the (1+2) in the lab. frame.
C* (Update Record)
C*    92/06/26  K.Fujii                Original version.
C*
CC**********************************************************************
 
      SUBROUTINE ELBXTR(EP,AMP,AMD,EMIN,EMAX)
 
      IMPLICIT   REAL*8 ( A-H, O-Z )
      REAL*8     EP, AMP, AMD, EMIN, EMAX
C
C==================< Entry Point >======================================
C
C--
C  Calculate beta and gamma.
C--
      GM   = EP/AMP
      BT   = SQRT( (GM-1)*(GM+1) )/GM
      X    = AMD/AMP
      PMX  = (AMP/2)*(1-X)*(1+X)
      EMX  = SQRT( PMX*PMX + AMD*AMD )
C--
C  Calculate Emin.
C--
      EMIN = EP - GM*( EMX + BT*PMX )
C--
C  Calculate Emax.
C--
      TEST = ( 1 + X*X )/2/X
      IF ( GM.LT.TEST ) THEN
         EMAX = EP - AMD
      ELSE
         EMAX = EP - GM*( EMX - BT*PMX )
      ENDIF
C--
C  That's it.
C--
      RETURN
      END
