C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   92/06/23  K.Fujii            Original version.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /PRJUNK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV), IDLP(2)
      REAL   *4       RBUF(MXxPT), EN(3,2)
      INTEGER*2       HBUF(2*MXxPT)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3), IBUF(MXxPT)
      EQUIVALENCE    (IDATA,RDATA), (RBUF,IBUF,HBUF)
C--
C  Define cuts.
C--
      DATA ELEPCT     /   5. /
C--
      DATA EVSLCT     /  20. /
      DATA DEVSCT     / 100. /
CX    DATA DEVSCT     / 250. /
      DATA PTLCUT     /   0. /
      DATA PTHCUT     / 999. /
C--
      DATA AMLCUT     / 10.0 /
      DATA CSLPCT     / 0.9  /
      DATA CSWMCT     / 0.75 /
      DATA ACOPCT     /  30. /
C--
C  Paritcle masses.
C--
      DATA AMZ        / 91.17 /
C     DATA AMT        / 150.  /
C     DATA AMW        / 80.0  /
C--
      DATA NCALL      / 0 /
C
C==================<<  Entry Point  >>================================
C
C--
C  Initialization.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         xPI    = ACOS(-1.)
         xRD2DG = 180/xPI
      ENDIF
CBGTEST       AMJCUT = -9999.
CBGTEST       DEVSCT = -9999.
CNOACOP       ACOPCT =  0.
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
      IRET = -10
C--
C  Cluster track matching.
C--
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  Set parent and daughter masses.
C--
      CALL TBGET(1,BKNAME(-2),1,NW,RDATA(1,1),IRT)
      AMP   = RDATA(3,1)
      CSLMG = RDATA(7,1)/UABSV3(RDATA(5,1))
      CALL TBGET(1,BKNAME(-2),3,NW,RDATA(1,1),IRT)
      AMD   = RDATA(3,1)
C--
C  TBGET combined trackinformation.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      CALL UVZERO(4,NTK)
      IBF = 1
      IBL = 3
      DO 1000 IB = IBF, IBL
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 1000
         DO 100 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
100      CONTINUE
1000  CONTINUE
C--
C  # tracks should be two.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1(2,NTK(1)+.1,1.)
C--
      IF ( NTK(1).NE.2 )                         GO TO 9999
      IF ( NTK(3).GT.0 )                         GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  First look for two energetic isolated leptons.
C--
      ELPLCT = ELEPCT
      ELPHCT = EBM - DEVSCT/2
C--
      IPT1 = NTK(0) + 1
      IPT2 = NTK(0) + NTK(1)
      NTRK = 0
      DO 2000 IPT = IPT1, IPT2
C--
C  Require high energy.
C--
         E = RDATA(4,IPT)
         CALL HCUM1( 3,E,1.)
         IF ( E.LT.ELPLCT .OR. E.GT.ELPHCT )     GO TO 2000
C--
         NTRK = NTRK + 1
         ICHG = RDATA( 9,IPT)
         ID   = RDATA(10,IPT)
         IDLP(NTRK) = ISIGN(ID,-ICHG)
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
2000  CONTINUE
C--
      IF ( NTRK.NE.2 )                           GO TO 9999
      CALL HCUM1( 4,NTRK+.1,1.)
      CALL HCUM1(1,2.1,1.)
C--
C  Charge balance.
C--
      IF ( IDLP(1)+IDLP(2).NE.0 )                GO TO 9999
      IF ( IDLP(1).LT.0 ) THEN
         IP = 1
         IM = 2
      ELSE
         IP = 2
         IM = 1
      ENDIF
C--
C  Require missing E and missing PT.
C--
      ISM = NTRK + 1
      CALL UADD4(QV(1,1),QV(1,2),QV(1,ISM))
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
      ELM  = QV(4,IM)
      ELP  = QV(4,IP)
      AMLL = UDOT4(QV(1,ISM),QV(1,ISM))
      AMLL = SIGN(SQRT(ABS(AMLL)),AMLL)
C--
      CALL HCUM2(201,EVIS,PL,1.)
      CALL HCUM1( 5,PT  ,1.)
C--
      EVSHCT = ROOTS - DEVSCT
      IF ( EVIS.LT.EVSLCT .OR. EVIS.GT.EVSHCT .OR.
     .     PT  .LT.PTLCUT .OR. PT  .GT.PTHCUT )  GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Require the 2-lepton system to have M_ll off M_Z.
C--
      CALL HCUM1(6,AMLL,1.)
C--
      IF ( ABS(AMLL-AMZ).LT.AMLCUT )             GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
C  Require the jets and the lepton be well measured.
C--
      CSLPM  =  QV(3,IM)/UABSV3(QV(1,IM))
      CSLPP  = -QV(3,IP)/UABSV3(QV(1,IP))
      CALL HCUM1(7,CSLPM,1.)
      CALL HCUM1(8,CSLPP,1.)
C--
      IF ( ABS(CSLPM).GT.CSLPCT .OR.
     .     ABS(CSLPP).GT.CSLPCT )                GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
C  cos(theta) cut again.
C--
      IF ( CSLPM.GT.CSWMCT .OR. CSLPP.GT.CSWMCT )  GO TO 9999
      CALL HCUM1(1,6.1,1.)
C--
C  Require a large acoplanarity.
C--
      APTLP1 = SQRT( QV(1,1)**2   + QV(2,1)**2   )
      APTLP2 = SQRT( QV(1,2)**2   + QV(2,2)**2   )
      ACOP  = ( QV(1,1)*QV(1,2) + QV(2,1)*QV(2,2) )/APTLP1/APTLP2
      ACOP  = 180 - ACOS(ACOP)*xRD2DG
      CALL HCUM1( 9,ACOP,1.)
C--
      IF ( ACOP.LT.ACOPCT )                      GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
C  Hists. and plots for selected sample.
C--
      ELPM = QV(4,IM)
      ELPP = QV(4,IP)
      CALL SOLVKN(ROOTS,AMP,AMD,QV(1,IM),QV(1,IP),EN,IRT)
      IF ( IRT.EQ.0 ) THEN
         IF ( QV(3,IM).LT.0. ) THEN
            CSLM1 = MIN(EN(3,1),EN(3,2))
            CSLM2 = MAX(EN(3,1),EN(3,2))
         ELSE
            CSLM1 = MAX(EN(3,1),EN(3,2))
            CSLM2 = MIN(EN(3,1),EN(3,2))
         ENDIF
      ELSE
         CSLM1 = -9999
         CSLM2 = -9999
      ENDIF
      CALL HCUM1( 21,AMLL ,1.)
      CALL HCUM1( 22,CSLPM,1.)
      CALL HCUM1( 23,CSLPP,1.)
      CALL HCUM1( 24,PT   ,1.)
      CALL HCUM1( 25,ACOP ,1.)
      CALL HCUM1( 26,ELPM ,1.)
      CALL HCUM1( 27,ELPP ,1.)
      CALL HCUM1( 28,ELPM ,1.)
      CALL HCUM1( 28,ELPP ,1.)
      CALL HCUM2(221,EVIS,PL  ,1.)
      CALL HCUM2(222,CSLM1,CSLMG,1.)
      CALL HCUM2(223,CSLM2,CSLMG,1.)
      CALL HCUM2(224,CSLM1,CSLM2,1.)
      IF ( ABS(CSLM1-CSLMG).GT.ABS(CSLM2-CSLMG) ) THEN
         CALL HCUM2(225,CSLM2,CSLM1,1.)
      ELSE
         CALL HCUM2(225,CSLM1,CSLM2,1.)
      ENDIF
C--
C  That's it.
C--
      IRET = 0
9999  RETURN
      END
