C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   92/06/23  K.Fujii            Original version.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /PRJUNK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV),
     .                NJCLS(MXxYCT), NPINJ(MXxJET), IPTOJ(MXxPV),
     .                PVJET(4,MXxPV), XHDATA(20,500),
     .                EIGVCT(0:3,3), ICHGJ(2), NCTKJ(2), IJI(2)
C--
      REAL   *4       RBUF(MXxPT)
      INTEGER*2       HBUF(2*MXxPT)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3), IBUF(MXxPT)
      EQUIVALENCE    (IDATA,RDATA), (RBUF,IBUF,HBUF)
C--
C  Define jet clustering parameters.
C--
      REAL*4          YCUT(0:MXxYCT)
      DATA   YCUT     / 4.0E-3,
     .                  0.5E-3, 1.0E-3, 1.5E-3, 2.0E-3, 2.5E-3,
     .                  3.0E-3, 3.5E-3, 4.0E-3, 4.5E-3, 5.0E-3,
     .                  5.5E-3, 6.0E-3, 6.5E-3, 7.0E-3, 7.5E-3,
     .                  8.0E-3, 8.5E-3, 9.0E-3, 9.5E-3, 1.0E-2 /
      DATA DYMAX      / 5.E-2/
      DATA NTRYMX     /  50  /
C--
C  Define cuts.
C--
      DATA NOMYCT     /  10  /
      DATA NTRKCT     /   4  /
      DATA ECUT       /  0.1 /
C--
      DATA EVSLCT     /  10. /
      DATA DEVSCT     /   0. /
      DATA PTLCUT     /   0. /
      DATA PTHCUT     / 999. /
      DATA ACOPCT     /  30. /
C--
      DATA AMJCUT     / 3.0  /
      DATA CSJTCT     / 0.9  /
      DATA THRSCT     / 0.99 /
C--
      DATA AMPI       / 0.13957 /
      DATA AME        / 0.000511/
      DATA AMU        / 0.10566 /
C--
      DATA NCALL      / 0 /
C
C==================<<  Entry Point  >>================================
C
C--
C  Initialization.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         xPI    = ACOS(-1.)
         xRD2DG = 180/xPI
      ENDIF
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
      IRET = -10
C--
C  Cluster track matching.
C--
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  TBGET combined trackinformation.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      CALL UVZERO(4,NTK)
      DO 1000 IB = 0, 3
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 1000
         DO 100 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
100      CONTINUE
1000  CONTINUE
C--
C  # tracks.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1(2,NTK(1)+NTK(2)+NTK(3)+.1,1.)
C--
      IF ( NPT.EQ.0 )                            GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  Select good tracks.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NPT
      NTRK = 0
      DO 2000 IPT = IPT1, IPT2
         E = RDATA(4,IPT)
         IF ( E.LT.ECUT )                        GO TO 2000
         NTRK = NTRK + 1
         CALL UVCOPY(10,RDATA(1,IPT),QV(1,NTRK))
         ICHG = RDATA( 9,IPT)
         ITYP = RDATA(10,IPT)
         IF ( ICHG.NE.0 ) THEN
            IF ( ITYP.EQ.11 ) THEN
               AM = AME
            ELSE IF ( ITYP.EQ.13 ) THEN
               AM = AMU
            ELSE
               AM = AMPI
            ENDIF
            CALL USETV4(AM,QV(1,NTRK),QV(1,NTRK))   
         ENDIF
2000  CONTINUE
C--
      CALL HCUM1(4,NTRK+.1,1.)
      IF ( NTRK.LT.NTRKCT )                      GO TO 9999
      CALL HCUM1(1,2.1,1.)
C--
C  Require missing E and missing PT.
C--
      ISM = NTRK + 1
      CALL UVZERO(4,QV(1,ISM))
      DO 300 ITK = 1, NTRK
         CALL UADD4(QV(1,ISM),QV(1,ITK),QV(1,ISM))
300   CONTINUE
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
C--
      CALL HCUM2(201,EVIS,PL,1.)
      CALL HCUM1( 7,PT  ,1.)
C--
      EVSHCT = ROOTS - DEVSCT
      IF ( EVIS.LT.EVSLCT .OR. EVIS.GT.EVSHCT .OR.
     .     PT  .LT.PTLCUT .OR. PT  .GT.PTHCUT )  GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Cluster into jets.
C--
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,MXxYCT,YCUT(1),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
C--
      IF ( IRT.LT.0 )                            GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
      ICTS = 0
      DO 4000 ICT = 1, MXxYCT
         IF ( NJCLS(ICT).GE.2 ) ICTS = ICT
         DO 400 IJT = 2, 10
            IHST = 10 + IJT
            IF ( NJCLS(ICT).EQ.IJT ) THEN
               EPS = 1.
            ELSE IF ( NJCLS(ICT).GT.10 .AND. IJT.EQ.10 ) THEN
               EPS = 1.
            ELSE
               EPS = 0.
            ENDIF
            CALL HCUM1(IHST,ICT+.1,EPS)
400      CONTINUE
4000  CONTINUE
C--
C  Require 2 or more jets for nominal y_cut.
C--
      IF ( NJCLS(NOMYCT).LT.2 )                  GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
C  Find a y_cut which gives 2 jets.
C--
      NTRY    = 0
C--
      IF ( NJCLS(ICTS).EQ.2 ) THEN
         YCUT(0) = YCUT(ICTS)
                                                 GO TO 460
      ELSE IF ( ICTS.LT.MXxYCT ) THEN
         YCUT(0) = YCUT(ICTS)
         DYCT    = (YCUT(ICTS+1)-YCUT(ICTS))/2
      ELSE
         YCUT(0) = YCUT(MXxYCT)
         DYCT    = DYMAX
      ENDIF
      NJT     = 3
450   NTRY    = NTRY + 1
      YCUT(0) = YCUT(0) + DYCT
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,1,YCUT(0),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
      IF ( IRT.LT.0 ) THEN
         PRINT *, ' >>>>>>> Error in JCLUST >>>>>>>'
         PRINT *, '    IRT = ', IRT, ' Skip this event. '
                                                 GO TO 9999
      ENDIF
      NJTS = NJT
      NJT  = NJCLS(1)
      IF ( NJT.EQ.2 )                            GO TO 460
      IF ( NTRY.GT.NTRYMX ) THEN
         PRINT *, ' >>>>>> Cannot find y_cut >>>>>> '
         PRINT *, '    NTRY    = ', NTRY
         PRINT *, '    YCUT(0) = ', YCUT(0), ' NJT = ', NJT
         CALL HCUM1(10,NTRY+.1,1.)
                                                 GO TO 9999
      ENDIF
      IF ( NJT.LT.2 ) THEN
         DYCT = - ABS(DYCT)/2
      ELSE IF ( NJT.GT.2 .AND. NJTS.LT.2 ) THEN
         DYCT = ABS(DYCT)/2
      ENDIF
                                                 GO TO 450
460   CALL HCUM1(10,NTRY+.1,1.)
      CALL HCUM1(11,YCUT(0),1.)
C--
      CALL HCUM1(1,6.1,1.)
C--
C  Store the reconstructed two jets in QV's.
C--
      IJ1 = ISM + 1
      IJ2 = IJ1 + 1
      IJI(1) = IJ1
      IJI(2) = IJ2
      CALL UVCOPY(4,PVJET(1,1),QV(1,IJ1))
      CALL UVCOPY(4,PVJET(1,2),QV(1,IJ2))
C--
      CALL UVZERO(2,NCTKJ)
      CALL UVZERO(2,ICHGJ)
      DO 500 ITK = 1, NTRK
         ICHG = QV(9,ITK)
         IF ( ICHG.NE.0 ) THEN
            IJ = IPTOJ(ITK)
            NCTKJ(IJ) = NCTKJ(IJ) + 1
            ICHGJ(IJ) = ICHGJ(IJ) + ICHG
         ENDIF
500   CONTINUE
C--
C  Require charge balance.
C--
      CALL HCUM2(202,ICHGJ(1)+.1,ICHGJ(2)+.1,1.)
      CALL HCUM2(203,NCTKJ(1)+.1,NCTKJ(2)+.1,1.)
      IF ( ICHGJ(1)*ICHGJ(2).GE.0 )		 GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
C  Require a large acoplanarity.
C--
      APTJ1 = SQRT( QV(1,IJ1)**2 + QV(2,IJ1)**2 )
      APTJ2 = SQRT( QV(1,IJ2)**2 + QV(2,IJ2)**2 )
      ACOP  = ( QV(1,IJ1)*QV(1,IJ2) + QV(2,IJ1)*QV(2,IJ2) )/APTJ1/APTJ2
      ACOP  = 180 - ACOS(ACOP)*xRD2DG
      CALL HCUM1( 9,ACOP,1.)
C--
      IF ( ACOP.LT.ACOPCT )                      GO TO 9999
      CALL HCUM1(1,8.1,1.)
C--
C  Require the two jets to have M_J consistent with tau.
C--
      AMJ1 = UDOT4(QV(1,IJ1),QV(1,IJ1))
      AMJ1 = SIGN(SQRT(ABS(AMJ1)),AMJ1)
      AMJ2 = UDOT4(QV(1,IJ2),QV(1,IJ2))
      AMJ2 = SIGN(SQRT(ABS(AMJ2)),AMJ2)
      CALL HCUM2(204,AMJ1,AMJ2,1.)
C--
      IF ( AMJ1.GT.AMJCUT .OR. AMJ2.GT.AMJCUT )	 GO TO 9999
      CALL HCUM1(1,9.1,1.)
C--
C  Require the jets be well measured.
C--
      CSJ1  = -ICHGJ(1)*QV(3,IJ1)/UABSV3(QV(1,IJ1))
      CSJ2  = -ICHGJ(2)*QV(3,IJ2)/UABSV3(QV(1,IJ2))
C--
      IF ( ABS(CSJ1).GT.CSJTCT .OR.
     .     ABS(CSJ2).GT.CSJTCT )                 GO TO 9999
      CALL HCUM1(1,10.1,1.)
C--
C  Thrust cut.
C--
      CALL THRUST(NTRK,QV(1,1),LNxPV,0,THR,OBL,EIGVCT)
      CALL HCUM1(27,THR,1.)
C--
      IF ( THR.GT.THRSCT )                       GO TO 9999
      CALL HCUM1(1,11.1,1.)
C--
C  Hists. and plots for selected sample.
C--
      CALL HCUM1(23,CSJ1,1.)
      CALL HCUM1(24,CSJ2,1.)
      CALL HCUM1(25,PT  ,1.)
      CALL HCUM1(26,ACOP,1.)
      CALL HCUM1(28,YCUT(0),1.)
      CALL HCUM2(223,EVIS,PL  ,1.)
      CALL HCUM2(224,AMJ1,AMJ2,1.)
      CALL HCUM2(225,QV(4,IJ1),QV(4,IJ2),1.)
C--
C  Look at 3-track system, if any.
C--
      DO 600 IJ = 1, 2
         NNUT = NPINJ(IJ) -  NCTKJ(IJ)               
	 EJET = QV(4,IJI(IJ))
         IF ( NCTKJ(IJ).EQ.3 ) THEN
            ICHJ = ICHGJ(IJ)
            EM12 = 9999.
            DO 60 ITK = 1, NTRK
               IF ( IPTOJ(ITK).EQ.IJ ) THEN
                  ICHG = QV(9,ITK)
                  IF ( ICHG.NE.0 .AND. ICHG.NE.ICHJ ) THEN
                     ITK3 = ITK
		     EM3  = QV(4,ITK)
                  ELSE
                     IF ( QV(4,ITK).LT.EM12 ) THEN
                        ITK12 = ITK
                        EM12  = QV(4,ITK)
                     ENDIF
                  ENDIF
               ENDIF
60          CONTINUE
	    CALL HCUM1(29,EJET,1.)
            CALL HCUM2(226,EM3/EBM,EM12/EBM,1.)
	    CALL HCUM2(227,EJET,NNUT+.1,1.)
            CALL HCUM2(229,EM3/EJET,EM12/EJET,1.)
         ELSE IF ( NCTKJ(IJ).EQ.1 ) THEN
	    CALL HCUM1(30,EJET,1.)
	    CALL HCUM2(228,EJET,NNUT+.1,1.)
         ENDIF
600   CONTINUE
C--
C  That's it.
C--
      IRET = 0
9999  RETURN
      END
