      IMPLICIT   REAL*8 ( A-H, O-Z )
      EP   = 125
      AMP  = 96.7
      AMD1 = 44.4
      AMD2 = 10
      CALL ELBXT2(EP,AMP,AMD1,AMD2,EMIN,EMAX)
      PRINT *, EMIN, EMAX
      END 
CC**********************************************************************
C*
C*====================================---------==
C* Subroutine ELBXT2(EP,AMP,AMD1,AMD2,EMIN,EMAX)
C*====================================---------==
C*
C* (Purpose)
C*    Calculate the minimum and maximum of the energy of the
C*    daughter D2 from the 2-body decay:
C*        P --> D1 + D2
C*    where the parent( P ) and the dauters have fixed masses.
C* (Inputs)
C*    EP    :(R*8): the parent E in the lab. frame.
C*    AMP   :(R*8): the parent mass.
C*    AMD1  :(R*8): the 1st daughter mass.
C*    AMD2  :(R*8): the 2nd daughter mass.
C* (Outputs)
C*    EMIN  :(R*8): the minimum E of the D2 in the lab. frame.
C*    EMAX  :(R*8): the maximum E of the D2 in the lab. frame.
C* (Update Record)
C*    92/06/26  K.Fujii                Original version.
C*
CC**********************************************************************
 
      SUBROUTINE ELBXT2(EP,AMP,AMD1,AMD2,EMIN,EMAX)
 
      IMPLICIT   REAL*8 ( A-H, O-Z )
      REAL*8     EP, AMP, AMD1, AMD2, EMIN, EMAX
C
C==================< Entry Point >======================================
C
C--
C  Calculate beta and gamma.
C--
      GM   = EP/AMP
      BT   = SQRT( (GM-1)*(GM+1) )/GM
      X1   = (AMD1/AMP)**2
      X2   = (AMD2/AMP)**2
      PMX  = (AMP/2)*SQRT( 1 - 2*(X1+X2) + (X1-X2)**2 )
      EMX  = SQRT( PMX*PMX + AMD2*AMD2 )
C--
C  Calculate Emin.
C--
      EMIN = GM*( EMX - BT*PMX )
C--
C  Calculate Emax.
C--
      EMAX = GM*( EMX + BT*PMX )
C--
C  That's it.
C--
      RETURN
      END
