C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   92/06/23  K.Fujii            Original version.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /PRJUNK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV),
     .                NJCLS(MXxYCT), NPINJ(MXxJET), IPTOJ(MXxPV),
     .                PVJET(4,MXxPV), XHDATA(20,500),
     .                EIGVCT(0:3,3)
      REAL   *4       RBUF(MXxPT)
      INTEGER*2       HBUF(2*MXxPT)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3), IBUF(MXxPT)
      EQUIVALENCE    (IDATA,RDATA), (RBUF,IBUF,HBUF)
C--
C  Define jet clustering parameters.
C--
      REAL*4          YCUT(0:MXxYCT)
      DATA   YCUT     / 4.0E-3,
     .                  0.5E-3, 1.0E-3, 1.5E-3, 2.0E-3, 2.5E-3,
     .                  3.0E-3, 3.5E-3, 4.0E-3, 4.5E-3, 5.0E-3,
     .                  5.5E-3, 6.0E-3, 6.5E-3, 7.0E-3, 7.5E-3,
     .                  8.0E-3, 8.5E-3, 9.0E-3, 9.5E-3, 1.0E-2 /
      DATA DYMAX      / 5.E-2/
      DATA NTRYMX     /  50  /
C--
C  Define cuts.
C--
      DATA NOMYCT     /  10  /
      DATA NTRKCT     /   5  /
      DATA ECUT       /  0.1 /
C--
      DATA ELEPCT     /   5. /
      DATA ECONCT     /   1. /
      DATA ECONTH     /  30. /
C--
      DATA EVSLCT     /  20. /
      DATA DEVSCT     / 100. /
      DATA PTLCUT     /   0. /
      DATA PTHCUT     / 999. /
      DATA ACOPCT     /  20. /
C     DATA ACOPCT     /  30. /
C     DATA ACOPCT     /   0. /
C--
      DATA AMJCUT     / 12.0 /
      DATA CSLPCT     / 0.9  /
      DATA CSJTCT     / 0.9  /
      DATA CSWMCT     / 0.75 /
      DATA THRSCT     / 0.99 /
C     DATA AMLCUT     /  0.0 /
      DATA AMLCUT     / 10.0 /
C--
C  Paritcle masses.
C--
      DATA AMW        / 80.0  /
C     DATA AMZ        / 91.17 /
C     DATA AMT        / 150.  /
C--
      DATA NCALL      / 0 /
C
C==================<<  Entry Point  >>================================
C
C--
C  Initialization.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         xPI    = ACOS(-1.)
         xRD2DG = 180/xPI
      ENDIF
CBGTEST       AMJCUT = -9999.
CBGTEST       AMLCUT = -9999.
CCBGTEST       DEVSCT = -9999.
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
      IRET = -10
C--
C  Cluster track matching.
C--
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  TBGET combined trackinformation.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      CALL UVZERO(4,NTK)
      DO 1000 IB = 0, 3
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 1000
         DO 100 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
100      CONTINUE
1000  CONTINUE
C--
C  # tracks.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1(2,NTK(1)+NTK(2)+NTK(3)+.1,1.)
C--
      IF ( NPT.EQ.0 )                            GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  First look for energetic isolated leptons.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NTK(0) + NTK(1)
      NLP  = 0
      NTRK = 0
      DO 2000 IPT = IPT1, IPT2
C--
C  Require high energy.
C--
         E = RDATA(4,IPT)
         CALL HCUM1( 5,E,1.)
         IF ( E.LT.ELEPCT )                      GO TO 2000
C--
C  Require isolation.
C        ECONCT
C        ECONTH
C--
         CALL GTECON(NTK(1)+NTK(2)+NTK(3),IPT-IPT1+1,LNxDT,
     .               RDATA(1,IPT1),ECONTH,ECN)
         CALL HCUM1( 6,ECN,1.)
         IF ( ECN.GT.ECONCT )                    GO TO 2000
C--
         NTRK = NTRK + 1
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
         ECON  = ECN
         ELP   = E
         IPTLP = IPT
         ICHG  = RDATA(9,IPT)
2000  CONTINUE
C--
C  Require one and only one energetic isolated lepton.
C--
      CALL HCUM1(3,NTRK+.1,1.)
      IF ( NTRK.NE.1 )                           GO TO 9999
      CALL HCUM1(1,2.1,1.)
C--
C  Select good tracks.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NPT
      NTRK = 1
      DO 3000 IPT = IPT1, IPT2
         E = RDATA(4,IPT)
         IF ( E.LT.ECUT )                        GO TO 3000
         IF ( IPT.EQ.IPTLP )                     GO TO 3000
         NTRK = NTRK + 1
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
3000  CONTINUE
C--
      CALL HCUM1(4,NTRK+.1,1.)
      IF ( NTRK.LT.NTRKCT )                      GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Require missing E and missing PT.
C--
      I2J = NTRK + 1
      CALL UVZERO(4,QV(1,I2J))
      DO 350 ITK = 2, NTRK
         CALL UADD4(QV(1,I2J),QV(1,ITK),QV(1,I2J))
350   CONTINUE
      ISM  = I2J + 1
      CALL UADD4(QV(1,I2J),QV(1,1),QV(1,ISM))
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
      EJJ  = QV(4,I2J)
      AMJJ = UDOT4(QV(1,I2J),QV(1,I2J))
      AMJJ = SIGN(SQRT(ABS(AMJJ)),AMJJ)
C--
      CALL HCUM2(204,EVIS,PL,1.)
      CALL HCUM2(205,AMJJ,EJJ,1.)
      CALL HCUM1( 7,PT  ,1.)
      CALL HCUM1( 8,AMJJ,1.)
C--
      EVSHCT = ROOTS - DEVSCT
      IF ( EVIS.LT.EVSLCT .OR. EVIS.GT.EVSHCT .OR.
     .     PT  .LT.PTLCUT .OR. PT  .GT.PTHCUT )  GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
C  Require a large acoplanarity.
C--
      APTLP = SQRT( QV(1,1)**2   + QV(2,1)**2   )
      APT2J = SQRT( QV(1,I2J)**2 + QV(2,I2J)**2 )
      ACOP  = ( QV(1,1)*QV(1,I2J) + QV(2,1)*QV(2,I2J) )/APTLP/APT2J
      IF ( ABS(ACOP).GT.1. ) ACOP = SIGN(1.,ACOP)
      ACOP  = 180 - ACOS(ACOP)*xRD2DG
      CALL HCUM1( 9,ACOP,1.)
C--
      IF ( ACOP.LT.ACOPCT )                      GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
C  Require the 2-jet system to have M_JJ less than M_W.
C--
      CALL HCUM1(21,AMJJ,1.)
      IF ( AMJJ.GT.AMW-AMJCUT )                  GO TO 9999
      CALL HCUM1(1,6.1,1.)
C--
C  Cluster into jets.
C--
      CALL JCLUST(NTRK-1,QV(1,2),LNxPV,0,MXxYCT,YCUT(1),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
C--
      IF ( IRT.LT.0 )                            GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
      ICTS = 0
      DO 4000 ICT = 1, MXxYCT
         IF ( NJCLS(ICT).GE.2 ) ICTS = ICT
         DO 400 IJT = 2, 10
            IHST = 10 + IJT
            IF ( NJCLS(ICT).EQ.IJT ) THEN
               EPS = 1.
            ELSE IF ( NJCLS(ICT).GT.10 .AND. IJT.EQ.10 ) THEN
               EPS = 1.
            ELSE
               EPS = 0.
            ENDIF
            CALL HCUM1(IHST,ICT+.1,EPS)
400      CONTINUE
4000  CONTINUE
C--
C  Require 2 or more jets for nominal y_cut.
C--
      IF ( NJCLS(NOMYCT).LT.2 )                  GO TO 9999
      CALL HCUM1(1,8.1,1.)
C--
C  Find a y_cut which gives 2 jets.
C--
      NTRY    = 0
C--
      IF ( NJCLS(ICTS).EQ.2 ) THEN
         YCUT(0) = YCUT(ICTS)
                                                 GO TO 460
      ELSE IF ( ICTS.LT.MXxYCT ) THEN
         YCUT(0) = YCUT(ICTS)
         DYCT    = (YCUT(ICTS+1)-YCUT(ICTS))/2
      ELSE
         YCUT(0) = YCUT(MXxYCT)
         DYCT    = DYMAX
      ENDIF
      NJT     = 3
450   NTRY    = NTRY + 1
      YCUT(0) = YCUT(0) + DYCT
      CALL JCLUST(NTRK-1,QV(1,2),LNxPV,0,1,YCUT(0),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
      IF ( IRT.LT.0 ) THEN
         PRINT *, ' >>>>>>> Error in JCLUST >>>>>>>'
         PRINT *, '    IRT = ', IRT, ' Skip this event. '
                                                 GO TO 9999
      ENDIF
      NJTS = NJT
      NJT  = NJCLS(1)
      IF ( NJT.EQ.2 )                            GO TO 460
      IF ( NTRY.GT.NTRYMX ) THEN
         PRINT *, ' >>>>>> Cannot find y_cut >>>>>> '
         PRINT *, '    NTRY    = ', NTRY
         PRINT *, '    YCUT(0) = ', YCUT(0), ' NJT = ', NJT
         CALL HCUM1(10,NTRY+.1,1.)
                                                 GO TO 9999
      ENDIF
      IF ( NJT.LT.2 ) THEN
         DYCT = - ABS(DYCT)/2
      ELSE IF ( NJT.GT.2 .AND. NJTS.LT.2 ) THEN
         DYCT = ABS(DYCT)/2
      ENDIF
                                                 GO TO 450
460   CALL HCUM1(10,NTRY+.1,1.)
      CALL HCUM1(11,YCUT(0),1.)
C--
      CALL HCUM1(1,9.1,1.)
C--
C  Require the jets and the lepton be well measured.
C--
      CSLP  = -ICHG*QV(3,1)/UABSV3(QV(1,1))
      CSJ1  =  ICHG*PVJET(3,1)/UABSV3(PVJET(1,1))
      CSJ2  =  ICHG*PVJET(3,2)/UABSV3(PVJET(1,2))
      CS2J  =  ICHG*QV(3,I2J)/UABSV3(QV(1,I2J))
C--
      IF ( ABS(CSLP).GT.CSLPCT .OR.
     .     ABS(CSJ1).GT.CSJTCT .OR.
     .     ABS(CSJ2).GT.CSJTCT )                 GO TO 9999
      CALL HCUM1(1,10.1,1.)
C--
C  Thrust cut.
C--
      CALL THRUST(NTRK,QV(1,1),LNxPV,0,THR,OBL,EIGVCT)
      CALL HCUM1(27,THR,1.)
C--
      IF ( THR.GT.THRSCT )                       GO TO 9999
      CALL HCUM1(1,11.1,1.)
C--
C  E_jet cut.
C--
C      EJJCUT = EBM - DEVSCT/2
C250
      EJJCUT = EBM - DEVSCT/4
      CALL HCUM1(33,EJJ,1.)
      IF ( EJJ.GT.EJJCUT )                       GO TO 9999
      CALL HCUM1(1,12.1,1.)
C--
C  M_lnu cut.
C--
         IWJ = ISM + 1
C>>>
C        CALL USETV4(AMW,QV(1,I2J),QV(1,IWJ))
         CALL UVCOPY(4,QV(1,I2J),QV(1,IWJ))
C>>>
         IVS = IWJ + 1
         CALL UADD4(QV(1,1),QV(1,IWJ),QV(1,IVS))
         EVSC = QV(4,IVS)
         PTC  = SQRT( QV(1,IVS)**2 + QV(2,IVS)**2 )
         PLC  = QV(3,IVS)
         AP   = SQRT( PTC**2 + PLC**2 )
C--
      PGM  = ( ROOTS - EVSC - AP )*( ROOTS - EVSC + AP )
     .       /2 /( PLC + ROOTS - EVSC )
      IF ( PGM.LT.0. ) PGM = 9999.
      PGP  = ( ROOTS - EVSC - AP )*( ROOTS - EVSC + AP )
     .       /2 /( PL - ROOTS + EVSC )
      IF ( PGP.GT.0. ) PGM = 9999.
      IF ( ABS(PGM).LT.ABS(PGP) ) THEN
         PG = PGM
      ELSE
         PG = PGP
      ENDIF
C--
      INU       = IVS + 1
      QV(1,INU) = -QV(1,IVS)
      QV(2,INU) = -QV(2,IVS)
      QV(3,INU) = -QV(3,IVS) - PG
      QV(4,INU) = ROOTS - EVSC - ABS(PG)
      IWL       = INU + 1
      CALL UADD4(QV(1,INU),QV(1,1),QV(1,IWL))
      AMWL      = UDOT4(QV(1,IWL),QV(1,IWL))
      AMWL      = SIGN(SQRT(ABS(AMWL)),AMWL)
C--
      CALL HCUM1(32,AMWL,1.)
C--
      IF ( ABS(AMWL-AMW).LT.AMLCUT )             GO TO 9999
      CALL HCUM1(1,13.1,1.)
C--
C  cos(theta) cut again.
C--
      IF ( CSLP.GT.CSWMCT .OR. CS2J.GT.CSWMCT )  GO TO 9999
      CALL HCUM1(1,14.1,1.)
C--
C  Hists. and plots for selected sample.
C--
      CALL HCUM1(22,AMJJ,1.)
      CALL HCUM1(23,CSLP,1.)
      CALL HCUM1(24,CS2J,1.)
      CALL HCUM1(25,PT  ,1.)
      CALL HCUM1(26,ACOP,1.)
      CALL HCUM1(28,YCUT(0),1.)
      CALL HCUM1(29,ECON,1.)
      CALL HCUM1(30,ELP ,1.)
      CALL HCUM1(31,EJJ ,1.)
      CALL HCUM2(223,EVIS,PL  ,1.)
      CALL HCUM2(224,AMJJ,AMWL,1.)
C--
C  That's it.
C--
      IRET = 0
9999  RETURN
      END
