C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   4/23/90  K.Fujii             Original version.
C*  12/15/92  K.Fujii             This version for WW mode.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /SSWORK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV),
     .                NJCLS(MXxYCT), NPINJ(MXxJET), IPTOJ(MXxPV),
     .                PVJET(4,MXxPV), XHDATA(20,500),
     .                EIGVCT(0:3,3)
      REAL   *4       RBUF(MXxPT), EN(3,2)
      INTEGER*2       HBUF(2*MXxPT)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3), IBUF(MXxPT)
      EQUIVALENCE    (IDATA,RDATA), (RBUF,IBUF,HBUF)
C--
      REAL*4          YCUT(0:MXxYCT)
C>>>
      DATA   YCUT     / 4.0E-3,
     .                  0.5E-3, 1.0E-3, 1.5E-3, 2.0E-3, 2.5E-3,
     .                  3.0E-3, 3.5E-3, 4.0E-3, 4.5E-3, 5.0E-3,
     .                  5.5E-3, 6.0E-3, 6.5E-3, 7.0E-3, 7.5E-3,
     .                  8.0E-3, 8.5E-3, 9.0E-3, 9.5E-3, 1.0E-2 /
      INTEGER*4       IUSFLG(MXxJET), IJTCMB(2,4,1000)
C>>>
      DATA NTRKCT     /  20  /
      DATA NOMYCT     /  10  /
      DATA DYMAX      / 1.E-2/
      DATA NTRYMX     /  50  /
      DATA ECUT       /  0.1 /
      DATA PTLCUT     /   0. /
      DATA PTHCUT     / 999. /
      DATA PLLCUT     /   0. /
      DATA PLHCUT     / 999. /
      DATA EVSLCT     /  20. /
      DATA DEVSCT     / 100. /
      DATA AMJCUT     / 20.0 /
      DATA ACOPCT     /  5.0 /
C--
      DATA AMW        / 80.0 /
      DATA SGMV       /   4. /
C>>>
      DATA NCALL      / 0 /
C
C==================<<  Entry Point  >>================================
C
C--
C  Initialization.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         xPI    = ACOS(-1.)
         xRD2DG = 180/xPI
      ENDIF
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
      IRET = 1
C--
C  Cluster track matching.
C--
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  Set parent and daughter masses.
C--
      CALL TBGET(1,BKNAME(-2),2,NW,RDATA(1,1),IRT)
      AMP   = RDATA(3,1)
      CSXMG = RDATA(7,1)/UABSV3(RDATA(5,1))
      CALL TBGET(1,BKNAME(-2),6,NW,RDATA(1,1),IRT)
      AMD   = RDATA(3,1)
      CALL TBGET(1,BKNAME(-2),7,NW,RDATA(1,1),IRT)
      CALL TBGET(1,BKNAME(-2),8,NW,RDATA(1,2),IRT)
      IWMGN = 10
      CALL UADD4(RDATA(5,1),RDATA(5,2),PV(1,IWMGN))
C--
C  TBGET simulator information.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      CALL UVZERO(4,NTK)
      IBF = 1
      IBL = 3
      DO 1000 IB = IBF, IBL
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 1000
         DO 100 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
100      CONTINUE
1000  CONTINUE
C--
C  # tracks.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1(2,NTK(1)+.1,1.)
      CALL HCUM1(3,NTK(2)+.1,1.)
      CALL HCUM1(4,NTK(3)+.1,1.)
C--
      IF ( NPT.EQ.0 )                            GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  Select good tracks.
C--
      IPT1 = 1
      IPT2 = NPT
      NTRK = 0
      DO 2000 IPT = IPT1, IPT2
         E = RDATA(4,IPT)
         IF ( E.LT.ECUT )                        GO TO 2000
         NTRK = NTRK + 1
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
2000  CONTINUE
C--
      CALL HCUM1(5,NTRK+.1,1.)
      IF ( NTRK.LT.NTRKCT )                      GO TO 9999
      CALL HCUM1(1,2.1,1.)
C--
C  Visible energy and missing momentum cuts.
C--
      ISM = NTRK + 1
      CALL UVZERO(4,QV(1,ISM))
      DO 250 ITK = 1, NTRK
         CALL UADD4(QV(1,ISM),QV(1,ITK),QV(1,ISM))
250   CONTINUE
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
      CALL HCUM2(201,EVIS,PL,1.)
      CALL HCUM1(  8,PT,1.)
C--
      EVSHCT = ROOTS - DEVSCT
      IF ( EVIS.LT.EVSLCT .OR. EVIS.GT.EVSHCT .OR.
     .     PT.LT.PTLCUT .OR. PT.GT.PTHCUT .OR.
     .     ABS(PL).LT.PLLCUT .OR. ABS(PL).GT.PLHCUT )
     .                                           GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Cluster into jets.
C--
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,MXxYCT,YCUT(1),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
C--
      IF ( IRT.LT.0 )                            GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
      ICTS = 0
      DO 3000 ICT = 1, MXxYCT
         IF ( NJCLS(ICT).GE.4 ) ICTS = ICT
         DO 300 IJT = 2, 10
            IHST = 10 + IJT
            IF ( NJCLS(ICT).EQ.IJT ) THEN
               EPS = 1.
            ELSE IF ( NJCLS(ICT).GT.10 .AND. IJT.EQ.10 ) THEN
               EPS = 1.
            ELSE
               EPS = 0.
            ENDIF
            CALL HCUM1(IHST,ICT+.1,EPS)
300      CONTINUE
3000  CONTINUE
C--
C  Require 4 or more jets for nominal y_cut.
C--
      IF ( NJCLS(NOMYCT).LT.4 )                  GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
C  Find a y_cut which gives 4 jets.
C--
      NTRY    = 0
C--
      IF ( ICTS.LT.MXxYCT ) THEN
         YCUT(0) = YCUT(ICTS)
         DYCT    = (YCUT(ICTS+1)-YCUT(ICTS))/2
      ELSE IF ( NJCLS(ICTS).EQ.4 ) THEN
         YCUT(0) = YCUT(ICTS)
                                                 GO TO 360
      ELSE
         YCUT(0) = YCUT(MXxYCT)
         DYCT    = DYMAX
      ENDIF
350   NTRY    = NTRY + 1
      YCUT(0) = YCUT(0) + DYCT
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,1,YCUT(0),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
      IF ( IRT.LT.0 ) THEN
         PRINT *, ' >>>>>>> Error in JCLUST >>>>>>>'
         PRINT *, '    IRT = ', IRT, ' Skip this event. '
                                                 GO TO 9999
      ENDIF
      NJT = NJCLS(1)
      IF ( NJT.EQ.4 )                            GO TO 360
      IF ( NTRY.GT.NTRYMX ) THEN
         IF ( NJT.EQ.4 )                         GO TO 360
         PRINT *, ' >>>>>> Cannot find y_cut >>>>>> '
         PRINT *, '    NTRY    = ', NTRY
         PRINT *, '    YCUT(0) = ', YCUT(0), ' NJT = ', NJT
         CALL HCUM1(9,NTRY+.1,1.)
                                                 GO TO 9999
      ENDIF
      IF ( NJT.LT.4 ) THEN
         DYCT = - ABS(DYCT)/2
      ELSE IF ( NJT.GE.4 ) THEN
         DYCT = ABS(DYCT)/2
      ENDIF
                                                 GO TO 350
360   CALL HCUM1(9,NTRY+.1,1.)
      CALL HCUM1(10,YCUT(0),1.)
C--
C  Look for W+ and W-.
C--
      CALL UVZERO(MXxJET,IUSFLG)
      NCMB = 0
      DO 6000 IJT1 = 1, 3
      DO 600 JJT1 = IJT1+1, 4
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         AMJ = UDOT4(PV(1,1),PV(1,1))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 600
         IUSFLG(IJT1) = 1
         IUSFLG(JJT1) = 1
      DO 5000 IJT2 = 1, 3
         IF ( IUSFLG(IJT2).NE.0 )                GO TO 5000
      DO 500 JJT2 = IJT2+1, 4
         IF ( IUSFLG(JJT2).NE.0 )                GO TO 500
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMJ = UDOT4(PV(1,2),PV(1,2))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 500
         IUSFLG(IJT2) = 1
         IUSFLG(JJT2) = 1
         NCMB = NCMB + 1
         IJTCMB(1,1,NCMB) = IJT1
         IJTCMB(2,1,NCMB) = JJT1
         IJTCMB(1,2,NCMB) = IJT2
         IJTCMB(2,2,NCMB) = JJT2
         IUSFLG(IJT2) = 0
         IUSFLG(JJT2) = 0
500   CONTINUE
5000  CONTINUE
         IUSFLG(IJT1) = 0
         IUSFLG(JJT1) = 0
600   CONTINUE
6000  CONTINUE
C--
C  Check if there is any combination left.
C--
      CALL HCUM1(11,NCMB+.1,1.)
      IF ( NCMB.LE.0 )                           GO TO 9999
      CALL HCUM1(1,6.1,1.)
C--
C  Calculate invariant masses, etc.
C--
      CHI2S  = 1.E20
      IWWBST = 0
      IWWBS2 = 0
      DO 7000 ICMB = 1, NCMB
         IJT1 = IJTCMB(1,1,ICMB)
         JJT1 = IJTCMB(2,1,ICMB)
         IJT2 = IJTCMB(1,2,ICMB)
         JJT2 = IJTCMB(2,2,ICMB)
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMW1 = UDOT4(PV(1,1),PV(1,1))
         AMW1 = SIGN(SQRT(ABS(AMW1)),AMW1)
         AMW2 = UDOT4(PV(1,2),PV(1,2))
         AMW2 = SIGN(SQRT(ABS(AMW2)),AMW2)
C>>>
         IF ( UABSV3(PV(1,1)).GT.1.E-5 ) THEN
            CALL UBSTFD(PVJET(1,IJT1),PV(1,1),PV(1,101))
            CTH1 = UNDOT3(PV(1,101),PV(1,1))
         ENDIF
         IF ( UABSV3(PV(1,2)).GT.1.E-5 ) THEN
            CALL UBSTFD(PVJET(1,IJT2),PV(1,2),PV(1,102))
            CTH2 = UNDOT3(PV(1,102),PV(1,2))
         ENDIF
C--
         CHI2 =   ((AMW1-AMW)/SGMV)**2 + ((AMW2-AMW)/SGMV)**2
         IF ( CHI2.LT.CHI2S ) THEN
            CHI2S  = CHI2
            IWWBS2 = IWWBST
            IWWBST = ICMB
         ENDIF
C--
         APTW1 = SQRT( PV(1,1)**2 + PV(2,1)**2 )
         APTW2 = SQRT( PV(1,2)**2 + PV(2,2)**2 )
         ACOP  = ( PV(1,1)*PV(1,2) + PV(2,1)*PV(2,2) )/APTW1/APTW2
         ACOP  = 180 - ACOS(ACOP)*xRD2DG
C-
         CALL UOPANG(PV(1,1),PV(1,IWMGN),DCSW1)
         CALL UOPANG(PV(1,2),PV(1,IWMGN),DCSW2)
         IF ( ABS(DCSW1).LT.ABS(DCSW2) ) THEN
            IM = 1
            IP = 2
         ELSE
            IM = 2
            IP = 1
         ENDIF
         EWM  = PV(4,IM)
         EWP  = PV(4,IP)
         CALL SOLVKN(ROOTS,AMP,AMD,PV(1,IM),PV(1,IP),EN,IRT)
         IF ( IRT.EQ.0 ) THEN
            IF ( PV(3,IM).LT.0. ) THEN
               CSXM1 = MIN(EN(3,1),EN(3,2))
               CSXM2 = MAX(EN(3,1),EN(3,2))
            ELSE
               CSXM1 = MAX(EN(3,1),EN(3,2))
               CSXM2 = MIN(EN(3,1),EN(3,2))
            ENDIF
         ELSE
            CSXM1 = -9999
            CSXM2 = -9999
         ENDIF
C-
         XHDATA( 1,ICMB) = ICMB
         XHDATA( 2,ICMB) = AMW1
         XHDATA( 3,ICMB) = AMW2
         XHDATA( 4,ICMB) = EVIS
         XHDATA( 5,ICMB) = PT
         XHDATA( 6,ICMB) = PL
         XHDATA( 7,ICMB) = CHI2
         XHDATA( 8,ICMB) = CTH1
         XHDATA( 9,ICMB) = CTH2
         XHDATA(10,ICMB) = ACOP
         XHDATA(11,ICMB) = CSXM1
         XHDATA(12,ICMB) = CSXM2
         CALL HCUM2(202,AMW1,AMW2,1.)
7000  CONTINUE
C--
C  Look at the best combination.
C--
      ICMB = IWWBST
      AMW1  = XHDATA( 2,ICMB)
      AMW2  = XHDATA( 3,ICMB)
      EVIS  = XHDATA( 4,ICMB)
      PT    = XHDATA( 5,ICMB)
      PL    = XHDATA( 6,ICMB)
      CHI2  = XHDATA( 7,ICMB)
      CTH1  = XHDATA( 8,ICMB)
      CTH2  = XHDATA( 9,ICMB)
      ACOP  = XHDATA(10,ICMB)
      CSXM1 = XHDATA(11,ICMB)
      CSXM2 = XHDATA(12,ICMB)
C--
C  Require a large acoplanarity.
C--
      CALL HCUM1(24,ACOP,1.)
      IF ( ACOP.LT.ACOPCT )                      GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
      CALL HCUM1(21,PT  ,1.)
      CALL HCUM1(22,CHI2,1.)
      CALL HCUM1(23,YCUT(0),1.)
      CALL HCUM2(221,EVIS,PL  ,1.)
      CALL HCUM2(222,AMW1 ,AMW2,1.)
      CALL HCUM2(223,ABS(CTH1),ABS(CTH2),1.)
      CALL HCUM2(224,CSXM1,CSXMG,1.)
      CALL HCUM2(225,CSXM2,CSXMG,1.)
      CALL HCUM2(226,CSXM1,CSXM2,1.)
      IF ( ABS(CSXM1-CSXMG).GT.ABS(CSXM2-CSXMG) ) THEN
         CALL HCUM2(227,CSXM2,CSXM1,1.)
      ELSE
         CALL HCUM2(227,CSXM1,CSXM2,1.)
      ENDIF
C--
C  That's it.
C--
9999  RETURN
      END
