CC**********************************************************************
C*
C*===============================----===
C* Subroutine FULCXX(IDP,IHEL,PV,AMP2)
C*===============================----===
C*
C* (Purpose)
C*    Calculate amplitude for e+ + e- ---> XC+ + XC-.
C* (Inputs)
C*       IDP(1,i) : (I*4) : generation number.
C*          (2,i) : (I*4) : T3 + 1.5.
C*          (3,i) : (I*4) : l/q flag; (1,2) = (l,q).
C*       IHEL( i) : (I*4) : helicity combination.
C*       PV(*, i) : (R*4) : 4-momentum of i-th particle.
C*                        : i =  1 for  e-.
C*                        :   =  2 for  e+.
C*                        :   =  3 for  LSP(snu) from XC+.
C*                        :   =  4 for  l+ from XC+
C*                        :   =  5 for  LSP(snu) from XC-.
C*                        :   =  6 for  l- from XC-.
C* (Outputs)
C*    AMP2(0)     : (R*4) : (amplitude sum)**2.
C*        (1)     : (R*4) : photon.
C*        (2)     : (R*4) : Z.
C*        (3)     : (R*4) : t-channel snu-exchange.
C* (Relation)
C*    Invokes AMPCXX and subroutines in HELAS.LOAD.
C* (Update Record)
C*    92/06/09  K.Fujii          Original version.
C*    96/02/16  K.Fujii          LSP=snu version.
C*
CC**********************************************************************
 
      SUBROUTINE FULCXX(IDP,IHEL,PV,AMP2)
 
      IMPLICIT   REAL*4  ( A-H, O-Z )
=EXPAND 'gen_lib/sm/SMCONS.inc'
=EXPAND 'gen_lib/ss/SSCONS.inc'
=EXPAND 'gen_lib/sm/SMPTAB.inc'
=EXPAND 'gen_lib/ss/SSPTAB.inc'
=EXPAND 'gen_lib/sm/SMCUPL.inc'
=EXPAND 'gen_lib/ss/SSCUPL.inc'
      INTEGER*4  IDP(3,*), IHEL(*)
      REAL   *4  PV(0:3,*), AMP2(0:3)
C--
      COMPLEX*8  SPINOR(6,6), EIN(6), EOT(6), XIN(6), XOT(6), AMP(0:3)
      COMPLEX*8  GSNX(2,2), GCFSF(2,2)
      REAL   *4  GAX(2), GZX(2)
      DATA NCALL / 0 /
C
C========< Entry Point >================================================
C
C--
C  Prepare coupling constants.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL     = 1
         AMSE      = AMF(1,2,1)
         AMSZ      = AMZ
         GAMZ      = GMZTOT
         GAX(1)    = GAL(1)
         GAX(2)    = GAL(2)
         GZX(1)    = GCCZ(1,1,1)
         GZX(2)    = GCCZ(2,1,1)
         GSNX(1,1) = GCESNL(1,1)
         GSNX(2,1) = GCESNL(2,1)
         GSNX(1,2) = CONJG(GCESNL(2,1))
         GSNX(2,2) = CONJG(GCESNL(1,1))
      ENDIF
C--
C  Set masses and couplings depending on final states.
C--
      AM3 = SFM(1)
      AM4 = AMF(IDP(1,4),IDP(2,4),IDP(3,4))
      AM5 = SFM(1)
      AM6 = AMF(IDP(1,6),IDP(2,6),IDP(3,6))
C--
      GCFSF(1,1) = GCESNL(1,1)
      GCFSF(2,1) = GCESNL(2,1)
C--
      GCFSF(1,2) = CONJG(GCESNL(2,1))
      GCFSF(2,2) = CONJG(GCESNL(1,1))
C--
C  Prepare spinors for external lines.
C--
      CALL IXXXXX(PV(0,1),AMSE  ,IHEL(1),+1,EIN)
      CALL OXXXXX(PV(0,2),AMSE  ,IHEL(2),-1,EOT)
C--
      CALL SXXXXX(PV(0,3),               +1,SPINOR(1, 3))
      CALL IXXXXX(PV(0,4),AM4   ,IHEL(4),-1,SPINOR(1, 4))
C--
      CALL SXXXXX(PV(0,5),               +1,SPINOR(1, 5))
      CALL OXXXXX(PV(0,6),AM6   ,IHEL(6),+1,SPINOR(1, 6))
C--
C  Prepare spinors for X+.
C--
      CALL FSIXXX(SPINOR(1,4),SPINOR(1,3),GCFSF(1,1),
     .            SWM(1),GMSW(1),XIN)
C--
C  Prepare spinors for X-.
C--
      CALL FSOXXX(SPINOR(1,6),SPINOR(1,5),GCFSF(1,2),
     .            SWM(1),GMSW(1),XOT)
C--
C  Calculate amplitude**2.
C--
      CALL AMPCXX (GAL,GAX,GZL,GZX,GSNX,SFM(1),GMSF(1),AMSZ,GAMZ,
     .             EIN,EOT,XIN,XOT,AMP)
      AMP2(0) = ABS(AMP(0))**2
      AMP2(1) = ABS(AMP(1))**2
      AMP2(2) = ABS(AMP(2))**2
      AMP2(3) = ABS(AMP(3))**2
C--
C  That's it.
C--
      RETURN
      END
