C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   4/23/90  K.Fujii             Original version.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /SSWORK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV), SV(LNxPV,20),
     .                NJCLS(MXxYCT), NPINJ(MXxJET), IPTOJ(MXxPV),
     .                PVJET(4,MXxPV), XHDATA(50,500),
     .                EIGVCT(0:3,3), SDATA(LNxDT,20),
     .                IPTDEX(MXxPV), IGNDEX(MXxPT), IBTAG(6)
      REAL   *4       RBUF(MXxPT)
      INTEGER*2       HBUF(2*MXxPT)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3), IBUF(MXxPT)
      EQUIVALENCE    (IDATA,RDATA), (RBUF,IBUF,HBUF)
      INTEGER*4       IUSFLG(MXxJET), IJTCMB(2,4,1000)
C
C ===========================
C    Cuts for 6J Selection
C ===========================
C
C--
C  Parameters for jet-clustering.
C--
      REAL*4          YCUT(0:MXxYCT)
      DATA   YCUT     / 0.5E-4,
     .                  2.0E-4, 2.5E-4, 3.5E-4, 4.0E-4, 4.5E-4,
     .                  5.0E-4, 5.5E-4, 6.0E-4, 6.5E-4, 7.0E-4,
     .                  7.5E-4, 8.0E-4, 8.5E-4, 9.0E-4, 9.5E-4,
     .                  1.0E-3, 1.5E-3, 2.0E-3, 2.5E-3, 3.0E-3 /
C--
      DATA NOMYCT     /   1  /
      DATA DYMAX      / 1.E-2/
      DATA NTRYMX     /  50  /
C--
C  Track selection.
C--
      DATA NTRKCT     /  20  /
      DATA ECUT       /  0.1 /
C--
C  Lepton cuts.
C--
      DATA ELPLCT     / 1.E9 /
C--
C  e+- veto cuts.
C--
      DATA EEVETO     / 100. /
      DATA THVETO     / 0.15 /
C--
C  Energy momentum balance.
C--
      DATA PTCUT      /  50. /
      DATA PLCUT      / 1.E9 /
      DATA DEVSCT     / 800. /
      DATA DEPLCT     /-1.E9 /
C--
      DATA EVOFF      /  10. /
C--
C  Mass cuts.
C--
      DATA AMJCUT     /  50. /
      DATA RMTCUT     /  50. /
C--
C  Angle cuts.
C--
      DATA CSJCUT     / 0.95 /
C--
      DATA PTMCUT     /   2. /
      DATA CSMJCT     / 999. /
C--
      DATA CSJJCT     / 999. /
C--
      DATA CSBBCT     / 999. /
C--
      DATA CSBWCT     / 999. /
      DATA RCBWCT     / 999. /
C--
C  B-tagging.
C--
      DATA NBTKCT     /    2 /
      DATA NBJLCT     /    2 /
      DATA NBJHCT     /    3 /
C--
C  Parameters for best candidate selection.
C--
      DATA AMW        / 80.0 /
C      DATA AMT        / 150. /
      DATA AMT        / 170. /
C--
      DATA SGMV       /   4. /
      DATA SGMT       /   6. /
C--
C  Event shape cut.
C--
      DATA THRSCT     / 1.00 /
C--
C  Cut on #ttbar candidates.
C--
      DATA NTTCUT     /  999 /
C--
C  Artificial cut knowing generator information.
C--
      DATA ICMBCT     /    0 /
C
C
C==================<<  Entry Point  >>================================
C
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
      IRET = -1
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  Match clusters and tracks.
C--
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
C--
C  TBGET Spring information.
C        SDATA(*, 1) = nu
C             (*, 2) = nu_bar
C             (*, 3) = t
C             (*, 4) = b
C             (*, 5) = W+
C             (*, 6) = f_D_bar
C             (*, 7) = f_U
C             (*, 8) = t_bar
C             (*, 9) = b_bar
C             (*,10) = W-
C             (*,11) = f_D
C             (*,12) = f_U_bar
C--
      NSP = 0
      IB  = -2
      CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
      DO 1000 IE = 1, NE
         NSP     = NSP + 1
         CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,SDATA(1,NSP),IRT)
         CALL UVCOPY(4,SDATA(5,NSP),SV(1,NSP))
1000  CONTINUE
C--
C  For non-ttbar MC, set SDATA to dummy values.
C--
      IF ( NSP.NE.12 ) THEN
         DO 150 ISP = 1, 10
            SV(1,ISP) =  0.
            SV(2,ISP) =  0.
            SV(3,ISP) = 20.
            SV(4,ISP) = 20.
150      CONTINUE
      ENDIF
C--
C  Look at generated event distribution.
C--
      APTG  = UABSV3(SV(1,3))
      CSTG  = SV(3,3)/APTG
      CBWG1 = UNDOT3(SV(1,9),SV(1,10))
      CBWG2 = UNDOT3(SV(1,4),SV(1, 5))
      CALL HCUM1(29,CBWG1,1.)
      CALL HCUM1(29,CBWG2,1.)
      CALL HCUM2(205,CSTG,APTG,1.)
C--
C  TBGET simulator information.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      NNU = 0
      CALL UVZERO(4,NTK)
      CALL UVZERO(MXxPT,IGNDEX)
      DO 2000 IB = 0, 3
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 2000
         DO 200 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
            IF ( IB.EQ.0 ) THEN
               IGNDEX(IELM(IE,IB)) = NPT
               ID  = RDATA(2,NPT)
               IDA = IABS(ID)
               IF ( IDA.EQ.12 .OR. IDA.EQ.14 .OR. IDA.EQ.16 )
     .              NNU = NNU + 1
            ENDIF
200      CONTINUE
2000  CONTINUE
C--
C  # tracks.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1( 2,NTK(1)+.1,1.)
      CALL HCUM1( 3,NTK(2)+.1,1.)
      CALL HCUM1( 4,NTK(3)+.1,1.)
      CALL HCUM1(38,NNU   +.1,1.)
C--
CNUVETOGN       IF ( NNU.GT.0 )                            GO TO 9999
      IF ( NPT.EQ.0 )                            GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  Select good clusters.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NPT
      NTRK = 0
      NLPT = 0
      NVTO = 0
      DO 220  IPT = IPT1, IPT2
         E    = RDATA( 4,IPT)
         ITYP = RDATA(10,IPT)
         IF ( E.LT.ECUT )                        GO TO 220
         NTRK = NTRK + 1
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
         IPTDEX(NTRK) = IPT
C--
         IF ( ITYP.EQ.11 .OR. ITYP.EQ.13 ) THEN
            IF ( E.GT.ELPLCT ) NLPT = NLPT + 1
            IF ( ITYP.EQ.11 ) THEN
               CS = QV(3,NTRK)/UABSV3(QV(1,NTRK))
               IF ( E.GT.EEVETO .AND. ABS(CS).GT.CSVETO ) THEN
                  NVTO = NVTO + 1
                  CALL HCUM2(209,CS,E,1.)
               ENDIF
            ENDIF
         ENDIF
220   CONTINUE
C--
      CALL HCUM1(5,NTRK+.1,1.)
      IF ( NTRK.LT.NTRKCT )                      GO TO 9999
      CALL HCUM1(1,2.1,1.)
      CALL HCUM1(31,NLPT+.1,1.)
      CALL HCUM1(40,NVTO+.1,1.)
      IF ( NVTO.GT.0 .OR. NLPT.GT.0 )            GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Require approximate 4-momentum conservation to select well
C  reconstructed events.
C--
      ISM = NTRK + 1
      ICM = NTRK + 2
      IMS = NTRK + 3
      CALL UVZERO(4,QV(1,ICM))
      QV(4,ICM) = ROOTS
      CALL UVZERO(4,QV(1,ISM))
      DO 250 ITK = 1, NTRK
         CALL UADD4(QV(1,ISM),QV(1,ITK),QV(1,ISM))
250   CONTINUE
      CALL USUB3(QV(1,ICM),QV(1,ISM),QV(1,IMS))
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
      DEPL = EVIS + EVOFF + ABS(PL) - ROOTS
      CALL HCUM2(204,EVIS,PL,1.)
      CALL HCUM1(  8,PT,1.)
      CALL HCUM2(208,DEPL,PT,1.)
C--
CB    EVSCUT = REVSCT*ROOTS
      EVSCUT = ROOTS - DEVSCT
      IF ( EVIS.LT.EVSCUT .OR. PT.LT.PTCUT .OR.
     .     ABS(PL).GT.PLCUT .OR. DEPL.LT.DEPLCT+EVOFF )	 GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
C  Cluster into jets.
C--
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,MXxYCT,YCUT(1),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
C--
      IF ( IRT.LT.0 )                            GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
      ICTS = 0
      DO 3000 ICT = 1, MXxYCT
         IF ( NJCLS(ICT).GE.6 ) ICTS = ICT
         DO 300 IJT = 2, 10
            IHST = 10 + IJT
            IF ( NJCLS(ICT).EQ.IJT ) THEN
               EPS = 1.
            ELSE IF ( NJCLS(ICT).GT.10 .AND. IJT.EQ.10 ) THEN
               EPS = 1.
            ELSE
               EPS = 0.
            ENDIF
            CALL HCUM1(IHST,ICT+.1,EPS)
300      CONTINUE
3000  CONTINUE
C--
C  Require 6 or more jets for nominal y_cut.
C--
      IF ( NJCLS(NOMYCT).LT.6 )                  GO TO 9999
      CALL HCUM1(1,6.1,1.)
C--
C  Find a y_cut which gives 6 jets.
C--
      NTRY    = 0
C--
      IF ( ICTS.LT.MXxYCT ) THEN
         YCUT(0) = YCUT(ICTS)
         DYCT    = (YCUT(ICTS+1)-YCUT(ICTS))/2
      ELSE IF ( NJCLS(ICTS).EQ.6 ) THEN
         YCUT(0) = YCUT(ICTS)
                                                 GO TO 360
      ELSE
         YCUT(0) = YCUT(MXxYCT)
         DYCT    = DYMAX
      ENDIF
350   NTRY    = NTRY + 1
      YCUT(0) = YCUT(0) + DYCT
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,1,YCUT(0),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
      IF ( IRT.LT.0 ) THEN
         PRINT *, ' >>>>>>> Error in JCLUST >>>>>>>'
         PRINT *, '    IRT = ', IRT, ' Skip this event. '
                                                 GO TO 9999
      ENDIF
      NJT = NJCLS(1)
      IF ( NJT.EQ.6 )                            GO TO 360
      IF ( NTRY.GT.NTRYMX ) THEN
         IF ( NJT.EQ.6 )                         GO TO 360
         PRINT *, ' >>>>>> Cannot find y_cut >>>>>> '
         PRINT *, '    NTRY    = ', NTRY
         PRINT *, '    YCUT(0) = ', YCUT(0), ' NJT = ', NJT
         CALL HCUM1(9,NTRY+.1,1.)
                                                 GO TO 9999
      ENDIF
      IF ( NJT.LT.6 ) THEN
         DYCT = - ABS(DYCT)/2
      ELSE IF ( NJT.GE.6 ) THEN
         DYCT = ABS(DYCT)/2
      ENDIF
                                                 GO TO 350
360   CALL HCUM1(9,NTRY+.1,1.)
      CALL HCUM1(10,YCUT(0),1.)
C--
C  Make sure jets are well within the detector acceptance.
C--
      CSJMX  = 0
      CSMJMX = 0
      DO 570 IJT = 1, 6
         CSJ = PVJET(3,IJT)/UABSV3(PVJET(1,IJT))
         IF ( ABS(CSJ).GT.ABS(CSJMX) ) CSJMX = CSJ
         CSMJ = UNDOT2(PVJET(1,IJT),QV(1,IMS))
         IF ( CSMJ.GT.CSMJMX ) CSMJMX = CSMJ
570   CONTINUE
      CALL HCUM1(30,CSJMX,1.)
      IF ( ABS(CSJMX).GT.CSJCUT )                GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
C  Cuts on angles between missing momentum and jets.
C--
      CALL HCUM1(33,CSMJMX,1.)
      IF ( PT.GT.PTMCUT ) CALL HCUM1(36,CSMJMX,1.)
      IF ( CSMJMX.GT.CSMJCT .AND. PT.GT.PTMCUT ) GO TO 9999
      CALL HCUM1(1,8.1,1.)
C--
C  Cuts on angles between jets.
C--
      CSJJMX = 0
      DO 580 IJT = 1, 5
         DO 58 JJT = IJT+1, 6
            CSJJ = UNDOT3(PVJET(1,IJT),PVJET(1,JJT))
            IF ( CSJJ.GT.CSJJMX ) CSJJMX = CSJJ
58       CONTINUE
580   CONTINUE
      CALL HCUM1(34,CSJJMX,1.)
      IF ( CSJJMX.GT.CSJJCT )                    GO TO 9999
      CALL HCUM1(1,9.1,1.)
C--
C  B-tagging.
C--
      NBJT = 0
      CALL UVZERO(6,IBTAG)
      DO 590 IJT = 1, 6
         CALL BTAGGN(NTRK,IJT,IPTOJ,IPTDEX,IGNDEX,LNxDT,
     .               RDATA(1,1),RDATA(1,1),NBTKCT,ISPT)
         IF ( ISPT.NE.0 ) THEN
            IBTAG(IJT) = 1
            NBJT       = NBJT + 1
         ENDIF
         IF ( NBJLCT.LE.1 ) IBTAG(IJT) = 1
590   CONTINUE
C--
      CALL HCUM1(39,NBJT+.1,1.)
      IF ( NBJT.LT.NBJLCT .OR. NBJT.GT.NBJHCT )  GO TO 9999
      CALL HCUM1(1,10.1,1.)
C--
C  Look for W+ and W-.
C--
      CALL UVZERO(MXxJET,IUSFLG)
      NCMB = 0
      DO 6000 IJT1 = 1, 5
      DO 600 JJT1 = IJT1+1, 6
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         AMJ = UDOT4(PV(1,1),PV(1,1))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 600
         IUSFLG(IJT1) = 1
         IUSFLG(JJT1) = 1
      DO 5000 IJT2 = 1, 5
         IF ( IUSFLG(IJT2).NE.0 )                GO TO 5000
      DO 500 JJT2 = IJT2+1, 6
         IF ( IUSFLG(JJT2).NE.0 )                GO TO 500
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMJ = UDOT4(PV(1,2),PV(1,2))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 500
C--
C  Check if b-jet candidates contain b-mesons.
C--
         IUSFLG(IJT2) = 1
         IUSFLG(JJT2) = 1
C--
         IBJ1 = 0
         IBJ2 = 0
         DO 50 IJT = 1, 6
            IF ( IUSFLG(IJT).EQ.0 ) THEN
               IF ( IBJ1.EQ.0 ) THEN
                  IBJ1 = IJT
               ELSE
                  IBJ2 = IJT
               ENDIF
            ENDIF
50       CONTINUE
C--
         DCSB11 = UNDOT3(PVJET(1,IBJ1),SV(1,4))
         DCSB22 = UNDOT3(PVJET(1,IBJ2),SV(1,9))
         DCSB12 = UNDOT3(PVJET(1,IBJ1),SV(1,9))
         DCSB21 = UNDOT3(PVJET(1,IBJ2),SV(1,4))
         CALL HCUM2(206,DCSB11,DCSB22,1.)
         CALL HCUM2(206,DCSB12,DCSB21,1.)
C--
         IUSFLG(IJT2) = 0
         IUSFLG(JJT2) = 0
C--
C        IF ( ( DCSB11.LT.DCBCT .OR. DCSB22.LT.DCBCT ) .AND.
C    .        ( DCSB21.LT.DCBCT .OR. DCSB12.LT.DCBCT ) ) GO TO 500
C--
C  B-tagging is effective only when NBJLCT > 1.
C--
         IF ( IBTAG(IBJ1).EQ.0 .OR. IBTAG(IBJ2).EQ.0 )   GO TO 500
C--
C  A candidate found. Store this.
C--
         IUSFLG(IJT2) = 1
         IUSFLG(JJT2) = 1
C--
         NCMB = NCMB + 1
         IJTCMB(1,1,NCMB) = IJT1
         IJTCMB(2,1,NCMB) = JJT1
         IJTCMB(1,2,NCMB) = IJT2
         IJTCMB(2,2,NCMB) = JJT2
         IJTCMB(1,4,NCMB) = IBJ1
         IJTCMB(2,4,NCMB) = IBJ2
C--
         IUSFLG(IJT2) = 0
         IUSFLG(JJT2) = 0
500   CONTINUE
5000  CONTINUE
         IUSFLG(IJT1) = 0
         IUSFLG(JJT1) = 0
600   CONTINUE
6000  CONTINUE
C--
C  Select correct jet assignment using generator information.
C--
      DCS2MN = 9999.
      DO 650 ICMB = 1, NCMB
         IJT1  = IJTCMB(1,1,ICMB)
         JJT1  = IJTCMB(2,1,ICMB)
         IBJ1  = IJTCMB(1,4,ICMB)
         IBJ2  = IJTCMB(2,4,ICMB)
C--
         DCS11 = UNDOT3(PVJET(1,IJT1),SV(1,12))
         DCS12 = UNDOT3(PVJET(1,JJT1),SV(1,11))
         DCS21 = UNDOT3(PVJET(1,IJT1),SV(1,11))
         DCS22 = UNDOT3(PVJET(1,JJT1),SV(1,12))
         DC21  = (DCS11-1)**2 + (DCS12-1)**2
         DC22  = (DCS21-1)**2 + (DCS22-1)**2
         DC2W1 = MIN(DC21,DC22)
C--
         DCS11 = UNDOT3(PVJET(1,IJT1),SV(1, 7))
         DCS12 = UNDOT3(PVJET(1,JJT1),SV(1, 6))
         DCS21 = UNDOT3(PVJET(1,IJT1),SV(1, 6))
         DCS22 = UNDOT3(PVJET(1,JJT1),SV(1, 7))
         DC21  = (DCS11-1)**2 + (DCS12-1)**2
         DC22  = (DCS21-1)**2 + (DCS22-1)**2
         DC2W2 = MIN(DC21,DC22)
C--
         IF ( DC2W1.LT.DC2W2 ) THEN
            DC2W = DC2W1
            DC2B = (UNDOT3(PVJET(1,IBJ1),SV(1,9))-1)**2 +
     .             (UNDOT3(PVJET(1,IBJ2),SV(1,4))-1)**2
         ELSE
            DC2W = DC2W2
            DC2B = (UNDOT3(PVJET(1,IBJ1),SV(1,4))-1)**2 +
     .             (UNDOT3(PVJET(1,IBJ2),SV(1,9))-1)**2
         ENDIF
         DC2 = DC2W + DC2B
         IF ( DC2.LT.DCS2MN ) THEN
            DCS2MN = DC2
            ICMBGN = ICMB
         ENDIF
650   CONTINUE
      CALL HCUM1(37,DCS2MN,1.)
C--
C  Too large DCS2MN means that none of the candidate combinations
C  are correct. In this case set ICMBGN to zero.
C--
      IF ( DCS2MN.GT.0.03 ) ICMBGN = 0
C--
C  Calculate invariant masses, etc.
C--
      CALL HCUM1(6,NCMB+.1,1.)
      IF ( NCMB.LE.0 )                           GO TO 9999
      CALL HCUM1(1,11.1,1.)
C>>>
CP       PRINT *, ' --- '
CP       DO 660 IJT = 1, 8
CP          PRINT *, ' PVJET(*,', IJT, ') = ', ( PVJET(K,IJT), K=1,4 )
CP 660   CONTINUE
CP       PRINT *, ' PVJET_sum = ', ( QV(K,ISM), K=1,4 )
CP       PRINT *, ' --- '
CP       PRINT *, ' NCMB = ', NCMB
C>>>
      NTT    = 0
      CHI2S  = 1.E20
      ITTBST = 0
      ITTBS2 = 0
      ICMBST = 0
      ICMBS2 = 0
      DO 7000 ICMB = 1, NCMB
C>>>
CP          PRINT *, ' ICMB = ', ICMB, ' IJTCMB = ',
CP      .             ( ( IJTCMB(K,KK,ICMB), K=1,2 ), KK=1,4 )
C>>>
         IJT1 = IJTCMB(1,1,ICMB)
         JJT1 = IJTCMB(2,1,ICMB)
         IJT2 = IJTCMB(1,2,ICMB)
         JJT2 = IJTCMB(2,2,ICMB)
         IBJ1 = IJTCMB(1,4,ICMB)
         IBJ2 = IJTCMB(2,4,ICMB)
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMW1 = UDOT4(PV(1,1),PV(1,1))
         AMW1 = SIGN(SQRT(ABS(AMW1)),AMW1)
         AMW2 = UDOT4(PV(1,2),PV(1,2))
         AMW2 = SIGN(SQRT(ABS(AMW2)),AMW2)
C--
         IF ( UABSV3(PV(1,1)).GT.1.E-5 ) THEN
            CALL UBSTFD(PVJET(1,IJT1),PV(1,1),PV(1,101))
            CTH1 = UNDOT3(PV(1,101),PV(1,1))
         ENDIF
         IF ( UABSV3(PV(1,2)).GT.1.E-5 ) THEN
            CALL UBSTFD(PVJET(1,IJT2),PV(1,2),PV(1,102))
            CTH2 = UNDOT3(PV(1,102),PV(1,2))
         ENDIF
C--
C>>> mW assignment.
C         CALL USETV4(AMW,PV(1,1),PV(1,4))
C         CALL USETV4(AMW,PV(1,2),PV(1,5))
         CALL UVCOPY(4,PV(1,1),PV(1,4))
         CALL UVCOPY(4,PV(1,2),PV(1,5))
C>>>
C--
         CALL UADD4(PV(1,4),PVJET(1,IBJ1),PV(1,6))
         CALL UADD4(PV(1,5),PVJET(1,IBJ2),PV(1,7))
         AMJ1 = UDOT4(PV(1,6),PV(1,6))
         AMJ1 = SIGN(SQRT(ABS(AMJ1)),AMJ1)
         AMJ2 = UDOT4(PV(1,7),PV(1,7))
         AMJ2 = SIGN(SQRT(ABS(AMJ2)),AMJ2)
         APT1 = UABSV3(PV(1,6))
         APT2 = UABSV3(PV(1,7))
C>>>
CP          PRINT *, ' AMT1, PV(*,T1) = ', AMJ1, ( PV(K,6), K=1,4 )
CP          PRINT *, ' AMT2, PV(*,T1) = ', AMJ2, ( PV(K,7), K=1,4 )
C>>>
         CALL HCUM2(201,AMJ1,AMJ2,1.)
C--
C  Cuts on b-W angles.
C--
         CSBW1 = UNDOT3(PVJET(1,IBJ1),PV(1,4))
         CSBW2 = UNDOT3(PVJET(1,IBJ2),PV(1,5))
         CALL HCUM2(207,CSBW1,CSBW2,1.)
         IF ( CSBW1.GT.CSBWCT .OR. CSBW2.GT.CSBWCT ) GO TO 7000
         RCBW = SQRT((CSBW1+1)**2+(CSBW2+1)**2)
         CALL HCUM1(35,RCBW,1.)
         IF ( RCBW.GT.RCBWCT )                       GO TO 7000
C--
C  Cuts on b-b angles.
C--
         CSBB  = UNDOT3(PVJET(1,IBJ1),PVJET(1,IBJ2))
         CSQQ1 = UNDOT3(PVJET(1,IJT1),PVJET(1,JJT1))
         CSQQ2 = UNDOT3(PVJET(1,IJT2),PVJET(1,JJT2))
C--
         IF ( ICMB.EQ.ICMBGN ) THEN
            CALL HCUM1(43,CSBB,1.)
         ELSE
            CALL HCUM1(53,CSBB,1.)
         ENDIF
         IF ( CSBB.GT.CSBBCT )                       GO TO 7000
C--
C  Mass cuts.
C--
         RMT = SQRT( (AMJ1-AMT)**2 + (AMJ2-AMT)**2 )
         IF ( RMT.LT.RMTCUT .OR.
     .      ( AMJ1.GE.AMT .AND. AMJ2.GE.AMT-RMTCUT ) .OR.
     .      ( AMJ2.GE.AMT .AND. AMJ1.GE.AMT-RMTCUT ) ) THEN
            NTT = NTT + 1
C--
            CHI2 =   ((AMW1-AMW)/SGMV)**2 + ((AMW2-AMW)/SGMV)**2
     .             + ((AMJ1-AMT)/SGMT)**2 + ((AMJ2-AMT)/SGMT)**2
            IF ( CHI2.LT.CHI2S ) THEN
               CHI2S  = CHI2
               ITTBS2 = ITTBST
               ITTBST = NTT
               ICMBS2 = ICMBST
               ICMBST = ICMB
            ENDIF
C--
            CALL UADD4(PV(1,6),PV(1,7),PV(1,10))
            AMTT  = UDOT4(PV(1,10),PV(1,10))
            AMTT  = SIGN(SQRT(ABS(AMTT)),AMTT)
            CST1  = PV(3,6)/UABSV3(PV(1,6))
            CST2  = PV(3,7)/UABSV3(PV(1,7))
C-
            XHDATA( 1,NTT) = ICMB
            XHDATA( 2,NTT) = AMW1
            XHDATA( 3,NTT) = AMW2
            XHDATA( 5,NTT) = AMJ1
            XHDATA( 6,NTT) = AMJ2
            XHDATA( 7,NTT) = AMTT
            XHDATA(10,NTT) = CST1
            XHDATA(11,NTT) = CST2
            XHDATA(12,NTT) = EVIS
            XHDATA(13,NTT) = PT
            XHDATA(14,NTT) = PL
            XHDATA(15,NTT) = CHI2
            XHDATA(16,NTT) = DEPL
            XHDATA(17,NTT) = CTH1
            XHDATA(18,NTT) = CTH2
            XHDATA(19,NTT) = APT1
            XHDATA(20,NTT) = APT2
            XHDATA(21,NTT) = CSBW1
            XHDATA(22,NTT) = CSBW2
            XHDATA(23,NTT) = CSBB
            XHDATA(24,NTT) = CSQQ1
            XHDATA(25,NTT) = CSQQ2
C--
            CALL HCUM1(7,AMTT,1.)
            CALL HCUM2(202,AMW1,AMW2,1.)
         ENDIF
7000  CONTINUE
C--
C  Check if there is any combination left.
C--
      CALL HCUM1(11,NTT+.1,1.)
      IF ( ITTBST.LE.0 )                         GO TO 9999
      CALL HCUM1(1,12.1,1.)
C--
C  Cut on # combinations.
C--
      IF ( NTT.GT.NTTCUT )                       GO TO 9999
      CALL HCUM1(1,13.1,1.)
C--
C  Thrust cut.
C--
      CALL THRUST(NTRK,QV(1,1),LNxPV,0,THR,OBL,EIGVCT)
      CALL HCUM1(27,THR,1.)
      IF ( THR.GT.THRSCT )                       GO TO 9999
      CALL HCUM1(1,14.1,1.)
C--
C  ICMBGN cut. This is an artificial cut to reject events for
C  for which no correct combinations exists.
C--
      IF ( ICMBGN.LT.ICMBCT )                    GO TO 9999
C>>>
CICMBGN       IF ( ICMBGN.NE.ICMBST )                    GO TO 9999
C>>>
      CALL HCUM1(1,15.1,1.)
C--
C  Look at the best combination.
C--
      ITT = ITTBST
      AMW1  = XHDATA( 2,ITT)
      AMW2  = XHDATA( 3,ITT)
      AMT1  = XHDATA( 5,ITT)
      AMT2  = XHDATA( 6,ITT)
      AMTT  = XHDATA( 7,ITT)
      CST1  = XHDATA(10,ITT)
      CST2  = XHDATA(11,ITT)
      EVIS  = XHDATA(12,ITT)
      PT    = XHDATA(13,ITT)
      PL    = XHDATA(14,ITT)
      CHI2  = XHDATA(15,ITT)
      DEPL  = XHDATA(16,ITT)
      CTH1  = XHDATA(17,ITT)
      CTH2  = XHDATA(18,ITT)
      APT1  = XHDATA(19,ITT)
      APT2  = XHDATA(20,ITT)
      CSBW1 = XHDATA(21,ITT)
      CSBW2 = XHDATA(22,ITT)
      IF ( ITTBS2.GT.0 ) THEN
         AMTT2 = XHDATA( 7,ITTBS2)
         CHI22 = XHDATA(15,ITTBS2)
      ELSE
         AMTT2 = 999.
         CHI22 = 999.
      ENDIF
C--
      DAPT1 = APT1 - APTG
      DAPT2 = APT2 - APTG
C--
      CALL HCUM1(21,AMW1,1.)
      CALL HCUM1(21,AMW2,1.)
      CALL HCUM1(23,AMTT,1.)
      CALL HCUM1(24,CST1,1.)
      CALL HCUM1(24,CST2,1.)
      CALL HCUM1(25,PT  ,1.)
      CALL HCUM1(26,CHI2,1.)
      CALL HCUM1(28,YCUT(0),1.)
      CALL HCUM1(32,APT1-APT2,1.)
      CALL HCUM2(221,AMT1,AMT2,1.)
      CALL HCUM2(223,EVIS,PL  ,1.)
      CALL HCUM2(228,DEPL ,PT,1.)
      CALL HCUM2(229,AMW1 ,AMW2,1.)
      CALL HCUM2(230,MIN(AMT1,AMT2),MAX(AMT1,AMT2),1.)
      CALL HCUM2(231,ABS(CTH1),ABS(CTH2),1.)
      CALL HCUM2(232,APT1,APT2,1.)
      CALL HCUM2(233,CSBW1,CSBW2,1.)
      CALL HCUM2(243,CSTG ,APTG ,1.)
      CALL HCUM2(244,CST1 ,APT1 ,1.)
      CALL HCUM2(244,CST2 ,APT2 ,1.)
      CALL HCUM2(249,APTG ,DAPT1,1.)
      CALL HCUM2(249,APTG ,DAPT2,1.)
      CALL HCUM2(250,REAL(ICMBGN)+.1,REAL(ICMBST)+.1,1.)
      CALL HCUM2(251,REAL(ICMBGN)+.1,REAL(ICMBS2)+.1,1.)
      CALL HCUM2(252,CSJJMX ,APT1,1.)
      CALL HCUM2(252,CSJJMX ,APT2,1.)
      CALL HCUM2(253,CSBB   ,APT1,1.)
      CALL HCUM2(253,CSBB   ,APT2,1.)
      CALL HCUM2(254,REAL(NBJT)+.1,APT1,1.)
      CALL HCUM2(254,REAL(NBJT)+.1,APT2,1.)
      IF ( ICMBGN.EQ.ICMBST ) THEN
         CALL HCUM1(41,CSJJMX ,1.)
         CALL HCUM1(42,REAL(NBJT)+.1,1.)
         CALL HCUM1(44,YCUT(0),1.)
         CALL HCUM2(261,CSBB ,APT1 ,1.)
         CALL HCUM2(261,CSBB ,APT2 ,1.)
         CALL HCUM2(262,CSQQ1,CSQQ2,1.)
         CALL HCUM2(263,AMW1 ,AMW2 ,1.)
         CALL HCUM2(264,AMT1 ,AMT2 ,1.)
      ELSE
         CALL HCUM1(51,CSJJMX,1.)
         CALL HCUM1(52,REAL(NBJT)+.1,1.)
         CALL HCUM1(54,YCUT(0),1.)
         CALL HCUM2(271,CSBB ,APT1 ,1.)
         CALL HCUM2(271,CSBB ,APT2 ,1.)
         CALL HCUM2(272,CSQQ1,CSQQ2,1.)
         CALL HCUM2(273,AMW1 ,AMW2 ,1.)
         CALL HCUM2(274,AMT1 ,AMT2 ,1.)
      ENDIF
C--
C  That's it.
C--
      IRET = 0
9999  RETURN
      END
 
      FUNCTION UNDOT2(A,B)
      DIMENSION A(2),B(2)
      AX    = SQRT(A(1)*A(1)+A(2)*A(2))
      BX    = SQRT(B(1)*B(1)+B(2)*B(2))
      UNDOT2 = ( A(1)*B(1) + A(2)*B(2) )/AX/BX
      RETURN
      END
