C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   4/23/90  K.Fujii             Original version.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /SSWORK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV),SV(LNxPV,20),
     .                NJCLS(MXxYCT), NPINJ(MXxJET), IPTOJ(MXxPV),
     .                PVJET(4,MXxPV), XHDATA(30,500),
     .                EIGVCT(0:3,3), RBUF(LNxDT,MXxPT), INCN(0:MXxPT),
     .                SDATA(LNxDT,20), IPTDEX(MXxPV), IGNDEX(MXxPT),
     .                IBTAG(4)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3)
      EQUIVALENCE    (IDATA,RDATA)
      INTEGER*4       IUSFLG(MXxJET), IJTCMB(2,4,1000)
C
C ===========================
C    Cuts for LJ Selection
C ===========================
C
C        CUT24   : Final cuts for ds/dp(tight mt cut version).
C        CUT25   : Final cuts for ds/dp(loose mt cut version).
C        CUT26   : Cuts for ds/dp(loose mt and no BJ).
C        CUT27   : Cuts for ds/dp(loose mt and new BJ version).
C        CUT28   : Cuts for ds/dp(loose mt and new BJ version).
C        CUT29   : Cuts for ds/dp(loose mt and new BJ version).
C        CUT30   : Cuts for ds/dp(loose mt and new BJ version).
C
C--
C  Parameters for jet clustering.
C--
      REAL*4          YCUT(0:MXxYCT)
      DATA   YCUT     / 4.0E-3,
     .                  5.0E-3, 5.5E-3, 6.0E-3, 6.5E-3, 7.0E-3,
     .                  7.5E-3, 8.0E-3, 8.5E-3, 9.0E-3,10.0E-3,
     .                 11.0E-3,12.0E-3,13.0E-3,14.0E-3,15.0E-3,
     .                 16.0E-3,17.0E-3,18.0E-3,19.0E-3,40.0E-3/
C--
      DATA NOMYCT     /  1   /
      DATA DYMAX      / 1.E-2/
      DATA NTRYMX     /  50  /
C--
C  Track selection.
C--
      DATA NTRKCT     /  20  /
      DATA ECUT       /  0.1 /
C--
C  Lepton cuts.
C--
      DATA ELEPCT     / 20.  /
      DATA ELPLCT     / 999. /
C--
      DATA ECONTH     / 20.  /
      DATA ECNTCT     /  1.5 /
      DATA EOPLCT     / 0.70 /
      DATA EOPHCT     / 1.30 /
C--
C  Energy and momentum balance.
C--
      DATA PTCUT      / 999. /
      DATA PLCUT      / 999. /
      DATA DEVSCT     / 999. /
      DATA EVOFF      / 10.0 /
C--
C  Mass cuts.
C--
C25-27,29DATA AMJCUT     /  12. /
C24,28,30
      DATA AMJCUT     /  20. /
C--
      DATA RMTCUT     / 50.0 /
C24   DATA AMTJCT     /   8. /
C24   DATA DELAMT     /  -4. /
C25-29DATA AMTJCT     /  20. /
C30
      DATA AMTJCT     /  15. /
      DATA DELAMT     /   0. /
C--
C  Angle cuts.
C--
      DATA CSJCUT     / 0.95 /
      DATA CSJJCT     / 999. /
      DATA CSBWCT     /-0.75 /
      DATA CSBBCT     / 0.70 /
C--
C  B-tagging.
C--
      DATA NBTKCT     /    2 /
C26   DATA NBJLCT     /    0 /
C24-25,27-30
      DATA NBJLCT     /    2 /
C28-30DATA NBJHCT     /    2 /
C24-27,31
      DATA NBJHCT     /  999 /
C--
C  Event shape cut.
C--
      DATA THRSCT     / 1.00 /
C--
C  Parameters for best candidate selection.
C--
      DATA AMW        / 80.0 /
C      DATA AMT        / 150. /
      DATA AMT        / 170. /
C--
      DATA SGMV       /   4. /
C--
C24   DATA SGMT       /   5. /
C25-30
      DATA SGMT       /  10. /
C
C==================<<  Entry Point  >>================================
C
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
      IRET = -1
C--
C  Cluster track matching.
C--
#ifdef USECMBTRK
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
#endif
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  TBGET Spring information.
C        SDATA(*, 1) = t_bar
C             (*, 2) = t
C             (*, 3) = b_bar
C             (*, 4) = W-
C             (*, 5) = b
C             (*, 6) = W+
C             (*, 7) = f_U_bar
C             (*, 8) = f_D
C             (*, 9) = f_U
C             (*,10) = f_D_bar
C--
      NSP = 0
      IB  = -2
      CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
      DO 1000 IE = 1, NE
         NSP     = NSP + 1
         CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,SDATA(1,NSP),IRT)
         CALL UVCOPY(4,SDATA(5,NSP),SV(1,NSP))
1000  CONTINUE
C--
C  For non-ttbar MC, set SDATA to dummy values.
C--
      IF ( NSP.NE.10 ) THEN
         DO 150 ISP = 1, 10
            SV(1,ISP) =  0.
            SV(2,ISP) =  0.
            SV(3,ISP) = 20.
            SV(4,ISP) = 20.
150      CONTINUE
      ENDIF
C--
C  Look at generated event distribution.
C--
      APTG  = UABSV3(SV(1,2))
      CSTG  = SV(3,2)/APTG
      CBWG1 = UNDOT3(SV(1,3),SV(1,4))
      CBWG2 = UNDOT3(SV(1,5),SV(1,6))
      CTHB1 = SV(3,3)/UABSV3(SV(1,3))
      CTHB2 = SV(3,5)/UABSV3(SV(1,5))
      CALL HCUM1(27,CBWG1,1.)
      CALL HCUM1(27,CBWG2,1.)
      CALL HCUM2(205,CSTG ,APTG ,1.)
      CALL HCUM2(207,CTHB1,CTHB2,1.)
C--
C  TBGET simulator information.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      CALL UVZERO(4,NTK)
      CALL UVZERO(MXxPT,IGNDEX)
      DO 2000 IB = 0, 3
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 2000
         DO 200 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
            IF ( IB.EQ.0 ) IGNDEX(IELM(IE,IB)) = NPT
200      CONTINUE
2000  CONTINUE
C--
C  # tracks.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1(2,NTK(1)+.1,1.)
      CALL HCUM1(3,NTK(2)+.1,1.)
      CALL HCUM1(4,NTK(3)+.1,1.)
C--
      IF ( NPT.EQ.0 )                            GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  First look for isolated leptons.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NTK(0) + NTK(1)
      NLP  = 0
      NTRK = 0
      DO 3000 IPT = IPT1, IPT2
C--
C  Require high energy.
C--
         E = RDATA(4,IPT)
         CALL HCUM1( 5,E,1.)
         IF ( E.GT.ELPLCT ) NLP = NLP + 1
         IF ( E.LT.ELEPCT )                      GO TO 3000
C--
C  Require no tracks nearby.
C        ECONCT
C        ECONTH
C--
         CALL GETECN(NTK(1)+NTK(2)+NTK(3),IPT-IPT1+1,LNxDT,
     .               RDATA(1,IPT1),ECONTH,ECNT,INCN(0))
         CALL HCUM1( 6,ECNT,1.)
         IF ( ECNT.GT.ECNTCT )                   GO TO 3000
C--
C  Look for calorimeter hits around this isolated track.
C--
         ECNC = RDATA(5,IPT)
         ECN  = ECNT + ECNC
         CALL HCUM2(201,E,ECNC,1.)
         CALL HCUM1( 7,ECN,1.)
         CALL HCUM1(28,ECNC/E,1.)
C--
C  Check particle ID.
C--
         ID   = RDATA(10,IPT)
         IDA  = IABS(ID)
         IF ( IDA.EQ.11 ) THEN
            EOP = ECNC/E
            CALL HCUM1(29,EOP,1.)
            IF ( EOP.LT.EOPLCT .OR. EOP.GT.EOPHCT ) GO TO 3000
         ELSE
            EOP   = ECN/E
         ENDIF
C--
C  Found an isolated lepton.
C--
         NTRK = NTRK + 1
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
         IPTDEX(NTRK) = IPT
         ELP   = E
         ECON  = ECN
         ITKLP = IPT
         ICHG  = RDATA(9,IPT)
3000  CONTINUE
C--
C  Require one and only one energetic isolated lepton.
C--
      CALL HCUM1(8,NTRK+.1,1.)
      IF ( NTRK.NE.1 .OR. NLP.GT.1 )             GO TO 9999
      CALL HCUM1(1,2.1,1.)
      ILP = 1
C--
C  Select good tracks.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NPT
      DO 4000 IPT = IPT1, IPT2
         E = RDATA(4,IPT)
         IF ( E.LT.ECUT .OR. IPT.EQ.ITKLP )      GO TO 4000
         NTRK = NTRK + 1
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
         IPTDEX(NTRK) = IPT
4000  CONTINUE
C--
      CALL HCUM1(9,NTRK-1+.1,1.)
      IF ( NTRK.LT.NTRKCT )                      GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Cuts on missing 4-momentum.
C--
      IMS = NTRK + 1
      ISM = NTRK + 2
      ICM = NTRK + 3
C--
      CALL UVZERO(4,QV(1,ICM))
      QV(4,ICM) = ROOTS
C--
      CALL UVZERO(4,QV(1,ISM))
      DO 450 ITK = 1, NTRK
         CALL UADD4(QV(1,ISM),QV(1,ITK),QV(1,ISM))
450   CONTINUE
C--
      CALL USUB4(QV(1,ICM),QV(1,ISM),QV(1,IMS))
C--
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
      CALL HCUM2(202,EVIS,PL,1.)
      CALL HCUM1( 10,PT,1.)
C--
      EVSCUT = ROOTS - DEVSCT
      IF ( EVIS.LT.EVSCUT .OR. PT.GT.PTCUT .OR.
     .     ABS(PL).GT.PLCUT )                    GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
C  Cluster into jets.
C--
      CALL JCLUST(NTRK-1,QV(1,2),LNxPV,0,MXxYCT,YCUT(1),
     .            NJCLS,NPINJ,IPTOJ(2),PVJET,IRT)
C--
      IF ( IRT.LT.0 )                            GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
      ICTS = 0
      DO 5000 ICT = 1, MXxYCT
         IF ( NJCLS(ICT).GE.4 ) ICTS = ICT
         DO 500 IJT = 2, 10
            IHST = 10 + IJT
            IF ( NJCLS(ICT).EQ.IJT ) THEN
               EPS = 1.
            ELSE IF ( NJCLS(ICT).GT.10 .AND. IJT.EQ.10 ) THEN
               EPS = 1.
            ELSE
               EPS = 0.
            ENDIF
            CALL HCUM1(IHST,ICT+.1,EPS)
500      CONTINUE
5000  CONTINUE
C--
C  Require 4 or more jets for nominal y_cut.
C--
      IF ( NJCLS(NOMYCT).LT.4 )                  GO TO 9999
      CALL HCUM1(1,6.1,1.)
C--
C  Find a y_cut which gives 4 jets.
C--
      NTRY    = 0
C--
      DYCT    = 0
      IF ( ICTS.LT.MXxYCT ) THEN
         YCUT(0) = YCUT(ICTS)
         DYCT    = (YCUT(ICTS+1)-YCUT(ICTS))/2
      ELSE IF ( NJCLS(ICTS).EQ.4 ) THEN
         YCUT(0) = YCUT(ICTS)
                                                 GO TO 560
      ELSE
         YCUT(0) = YCUT(MXxYCT)
         DYCT    = DYMAX
      ENDIF
550   NTRY    = NTRY + 1
      YCUT(0) = YCUT(0) + DYCT
      CALL JCLUST(NTRK-1,QV(1,2),LNxPV,0,1,YCUT(0),
     .            NJCLS,NPINJ,IPTOJ(2),PVJET,IRT)
      IF ( IRT.LT.0 ) THEN
         PRINT *, ' >>>>>>> Error in JCLUST >>>>>>>'
         PRINT *, '    IRT = ', IRT, ' Skip this event. '
                                                 GO TO 9999
      ENDIF
      NJT = NJCLS(1)
      IF ( NJT.EQ.4 .AND. ABS(DYCT).LT.1.E-4 )   GO TO 560
      IF ( NTRY.GT.NTRYMX ) THEN
         IF ( NJT.EQ.4 )                         GO TO 560
         PRINT *, ' >>>>>> Cannot find y_cut >>>>>> '
         PRINT *, '    NTRY    = ', NTRY
         PRINT *, '    YCUT(0) = ', YCUT(0), ' NJT = ', NJT
         CALL HCUM1(9,NTRY+.1,1.)
                                                 GO TO 9999
      ENDIF
      IF ( NJT.LT.4 ) THEN
         DYCT = - ABS(DYCT)/2
      ELSE IF ( NJT.GE.4 ) THEN
         DYCT = ABS(DYCT)/2
      ENDIF
                                                 GO TO 550
C--
C  y_cut found.
C--
560   CALL HCUM1(21,NTRY+.1,1.)
      CALL HCUM1(22,YCUT(0),1.)
C--
C  Store the isolated lepton as 5-th jet and the missing 4-momentum
C  which is presumably a neutrino as 6-th jet.
C--
      IPTOJ(1) = 5
      CALL UVCOPY(4,QV(1,ILP),PVJET(1,5))
      CALL UVCOPY(4,QV(1,IMS),PVJET(1,6))
C--
C  Make sure jets are well within the detector acceptance.
C--
      CSJMX = 0
      DO 570 IJT = 1, 4
         CSJ = PVJET(3,IJT)/UABSV3(PVJET(1,IJT))
         IF ( ABS(CSJ).GT.ABS(CSJMX) ) THEN
            CSJMX = CSJ
         ENDIF
570   CONTINUE
      CALL HCUM1(30,CSJMX,1.)
      IF ( ABS(CSJMX).GT.CSJCUT )                GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
C  Cuts on angles between jets.
C--
      CSJJMX = 0
      DO 580 IJT = 1, 3
         DO 58 JJT = IJT+1, 4
            CSJJ = UNDOT3(PVJET(1,IJT),PVJET(1,JJT))
            IF ( CSJJ.GT.CSJJMX ) CSJJMX = CSJJ
58       CONTINUE
580   CONTINUE
      CALL HCUM1(40,CSJJMX,1.)
      IF ( CSJJMX.GT.CSJJCT )                    GO TO 9999
      CALL HCUM1(1,8.1,1.)
C--
C  B-tagging.
C--
      NBJT = 0
      CALL UVZERO(4,IBTAG)
      DO 590 IJT = 1, 4
         CALL BTAGGN(NTRK,IJT,IPTOJ,IPTDEX,IGNDEX,LNxDT,
     .               RDATA(1,1),RDATA(1,1),NBTKCT,ISPT)
         IF ( ISPT.NE.0 ) THEN
            IBTAG(IJT) = ISPT
            NBJT       = NBJT + 1
         ENDIF
         IF ( NBJLCT.LE.1 ) IBTAG(IJT) = 1
590   CONTINUE
C--
      CALL HCUM1(39,NBJT+.1,1.)
      IF ( NBJT.LT.NBJLCT .OR. NBJT.GT.NBJHCT )  GO TO 9999
      CALL HCUM1(1,9.1,1.)
C--
C  Look for W+ or W-.
C--
      CALL UVZERO(MXxJET,IUSFLG)
      NCMB = 0
      DO 7000 IJT1 = 5, 5
      DO 700 JJT1 =  6, 6
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         AMJ = UDOT4(PV(1,1),PV(1,1))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
C--
C  Select W-->lnu candidates.
C--
C>>>
C1-4     IF ( ABS(AMJ-AMW).GT.2*AMJCUT )         GO TO 700
C1-16    IF ( ABS(AMJ-AMW).GT.3*AMJCUT )         GO TO 700
C17-->
         IF ( ABS(AMJ-AMW).GT.5*AMJCUT )         GO TO 700
C>>>
         IUSFLG(IJT1) = 1
         IUSFLG(JJT1) = 1
      DO 6000 IJT2 = 1, 3
         IF ( IUSFLG(IJT2).NE.0 )                GO TO 6000
      DO 600 JJT2 =  IJT2+1, 4
         IF ( IUSFLG(JJT2).NE.0 )                GO TO 600
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMJ = UDOT4(PV(1,2),PV(1,2))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
C--            _
C  Select W-->qq candidates.
C--
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 600
C--
C  Check if b-jet candidates contain b-mesons.
C--
         IUSFLG(IJT2) = 1
         IUSFLG(JJT2) = 1
C--
         IBJ1 = 0
         IBJ2 = 0
         DO 60 IJT = 1, 6
            IF ( IUSFLG(IJT).EQ.0 ) THEN
               IF ( IBJ1.EQ.0 ) THEN
                  IBJ1 = IJT
               ELSE
                  IBJ2 = IJT
               ENDIF
            ENDIF
60       CONTINUE
C--
         DCSB11 = UNDOT3(PVJET(1,IBJ1),SV(1,3))
         DCSB22 = UNDOT3(PVJET(1,IBJ2),SV(1,5))
         DCSB12 = UNDOT3(PVJET(1,IBJ1),SV(1,5))
         DCSB21 = UNDOT3(PVJET(1,IBJ2),SV(1,3))
         CALL HCUM2(206,DCSB11,DCSB22,1.)
         CALL HCUM2(206,DCSB12,DCSB21,1.)
C--
         IUSFLG(IJT2) = 0
         IUSFLG(JJT2) = 0
C--
C        IF ( ( DCSB11.LT.DCBCT .OR. DCSB22.LT.DCBCT ) .AND.
C    .        ( DCSB21.LT.DCBCT .OR. DCSB12.LT.DCBCT ) ) GO TO 600
C--
         IF ( IBTAG(IBJ1).EQ.0 .OR. IBTAG(IBJ2).EQ.0 )   GO TO 600
C--
C  A candidate found. Store this.
C--
         IUSFLG(IJT2) = 1
         IUSFLG(JJT2) = 1
C--
         NCMB = NCMB + 1
         IJTCMB(1,1,NCMB) = IJT1
         IJTCMB(2,1,NCMB) = JJT1
         IJTCMB(1,2,NCMB) = IJT2
         IJTCMB(2,2,NCMB) = JJT2
         IJTCMB(1,4,NCMB) = IBJ1
         IJTCMB(2,4,NCMB) = IBJ2
C--
         NCMB = NCMB + 1
         IJTCMB(1,1,NCMB) = IJT1
         IJTCMB(2,1,NCMB) = JJT1
         IJTCMB(1,2,NCMB) = IJT2
         IJTCMB(2,2,NCMB) = JJT2
         IJTCMB(1,4,NCMB) = IBJ2
         IJTCMB(2,4,NCMB) = IBJ1
C--
         IUSFLG(IJT2) = 0
         IUSFLG(JJT2) = 0
600   CONTINUE
6000  CONTINUE
         IUSFLG(IJT1) = 0
         IUSFLG(JJT1) = 0
700   CONTINUE
7000  CONTINUE
C--
C  Select correct jet assignment using generator information.
C--
      DCS2MN = 9999.
      DO 750 ICMB = 1, NCMB
         IJT2  = IJTCMB(1,2,ICMB)
         JJT2  = IJTCMB(2,2,ICMB)
         IBJ1  = IJTCMB(1,4,ICMB)
         IBJ2  = IJTCMB(2,4,ICMB)
C--
         IF ( ICHG.LT.0 ) THEN
            DC2B1 = (UNDOT3(PVJET(1,IBJ1),SV(1, 3))-1)**2
            DC2B2 = (UNDOT3(PVJET(1,IBJ2),SV(1, 5))-1)**2
            DCJ11 = UNDOT3(PVJET(1,IJT2),SV(1, 9))
            DCJ12 = UNDOT3(PVJET(1,JJT2),SV(1,10))
            DCJ21 = UNDOT3(PVJET(1,IJT2),SV(1,10))
            DCJ22 = UNDOT3(PVJET(1,JJT2),SV(1, 9))
            DC21  = (DCJ11-1)**2 + (DCJ12-1)**2
            DC22  = (DCJ21-1)**2 + (DCJ22-1)**2
            DC2WJ = MIN(DC21,DC22)
         ELSE
            DC2B1 = (UNDOT3(PVJET(1,IBJ1),SV(1, 5))-1)**2
            DC2B2 = (UNDOT3(PVJET(1,IBJ2),SV(1, 3))-1)**2
            DCJ11 = UNDOT3(PVJET(1,IJT2),SV(1, 7))
            DCJ12 = UNDOT3(PVJET(1,JJT2),SV(1, 8))
            DCJ21 = UNDOT3(PVJET(1,IJT2),SV(1, 8))
            DCJ22 = UNDOT3(PVJET(1,JJT2),SV(1, 7))
            DC21  = (DCJ11-1)**2 + (DCJ12-1)**2
            DC22  = (DCJ21-1)**2 + (DCJ22-1)**2
            DC2WJ = MIN(DC21,DC22)
         ENDIF
C--
         DC2 = DC2WJ + DC2B1 + DC2B2
         IF ( DC2.LT.DCS2MN ) THEN
            DCS2MN = DC2
            ICMBGN = ICMB
         ENDIF
750   CONTINUE
      CALL HCUM1(38,DCS2MN,1.)
C--
C  Calculate invariant masses, etc.
C--
      CALL HCUM1(23,NCMB+.1,1.)
      IF ( NCMB.LE.0 )                           GO TO 9999
      CALL HCUM1(1,10.1,1.)
C>>>
CP       PRINT *, ' --- '
CP       DO 660 IJT = 1, 8
CP          PRINT *, ' PVJET(*,', IJT, ') = ', ( PVJET(K,IJT), K=1,4 )
CP 660   CONTINUE
CP       PRINT *, ' PVJET_sum = ', ( QV(K,ISM), K=1,4 )
CP       PRINT *, ' --- '
CP       PRINT *, ' NCMB = ', NCMB
C>>>
      NTT    = 0
      CHI2S  = 1.E20
      ITTBST = 0
      ITTBS2 = 0
      ICMBST = 0
      ICMBS2 = 0
      DO 8000 ICMB = 1, NCMB
C>>>
CP          PRINT *, ' ICMB = ', ICMB, ' IJTCMB = ',
CP      .             ( ( IJTCMB(K,KK,ICMB), K=1,2 ), KK=1,4 )
C>>>
         IJT1 = IJTCMB(1,1,ICMB)
         JJT1 = IJTCMB(2,1,ICMB)
         IJT2 = IJTCMB(1,2,ICMB)
         JJT2 = IJTCMB(2,2,ICMB)
         IBJ1 = IJTCMB(1,4,ICMB)
         IBJ2 = IJTCMB(2,4,ICMB)
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMW1 = UDOT4(PV(1,1),PV(1,1))
         AMW1 = SIGN(SQRT(ABS(AMW1)),AMW1)
         AMW2 = UDOT4(PV(1,2),PV(1,2))
         AMW2 = SIGN(SQRT(ABS(AMW2)),AMW2)
C>>>
         IF ( UABSV3(PV(1,1)).GT.1.E-5 ) THEN
            CALL UBSTFD(PVJET(1,IJT1),PV(1,1),PV(1,101))
            IF ( UABSV3(PV(1,101)).GT.1.E-5 ) THEN
               CTH1 = UNDOT3(PV(1,101),PV(1,1))
            ELSE
               CTH1 = 9999
            ENDIF
         ENDIF
         IF ( UABSV3(PV(1,2)).GT.1.E-5 ) THEN
            CALL UBSTFD(PVJET(1,IJT2),PV(1,2),PV(1,102))
            IF ( UABSV3(PV(1,102)).GT.1.E-5 ) THEN
               CTH2 = UNDOT3(PV(1,102),PV(1,2))
            ELSE
               CTH2 = 9999
            ENDIF
         ENDIF
C>>>
C        CALL USETV4(AMW,PV(1,1),PV(1,4))
C        CALL USETV4(AMW,PV(1,2),PV(1,5))
         CALL UVCOPY(4,PV(1,1),PV(1,4))
         CALL UVCOPY(4,PV(1,2),PV(1,5))
C>>>
C--
         CALL UADD4(PV(1,4),PVJET(1,IBJ1),PV(1,6))
         CALL UADD4(PV(1,5),PVJET(1,IBJ2),PV(1,7))
         AMJ1 = UDOT4(PV(1,6),PV(1,6))
         AMJ1 = SIGN(SQRT(ABS(AMJ1)),AMJ1)
         AMJ2 = UDOT4(PV(1,7),PV(1,7))
         AMJ2 = SIGN(SQRT(ABS(AMJ2)),AMJ2)
         APT1 = UABSV3(PV(1,6))
         APT2 = UABSV3(PV(1,7))
C>>>
CP          PRINT *, ' AMT1, PV(*,T1) = ', AMJ1, ( PV(K,6), K=1,4 )
CP          PRINT *, ' AMT2, PV(*,T1) = ', AMJ2, ( PV(K,7), K=1,4 )
C>>>
         CALL HCUM2(203,AMJ1,AMJ2,1.)
         RMT = SQRT( (AMJ1-AMT)**2 + (AMJ2-AMT)**2 )
C--
C  Mass cuts.
C--
         IF ( ( RMT.GT.RMTCUT .AND.
     .        ( AMJ1.LT.AMT .OR. AMJ2.LT.AMT-RMTCUT ) .AND.
     .        ( AMJ2.LT.AMT .OR. AMJ1.LT.AMT-RMTCUT ) )
     .          .OR. ABS(AMJ2-AMT-DELAMT).GT.AMTJCT )  GO TO 8000
C--
         CALL UADD4(PV(1,6),PV(1,7),PV(1,10))
         AMTT = UDOT4(PV(1,10),PV(1,10))
         AMTT = SIGN(SQRT(ABS(AMTT)),AMTT)
         CST1 =  ICHG*PV(3,6)/UABSV3(PV(1,6))
         CST2 = -ICHG*PV(3,7)/UABSV3(PV(1,7))
         IF ( ICHG.LT.0 ) THEN
            DCST = UNDOT3(PV(1,7),SV(1,2))
            DCSW = UNDOT3(PV(1,5),SV(1,6))
            DCSB = UNDOT3(PVJET(1,IBJ2),SV(1,5))
         ELSE
            DCST = UNDOT3(PV(1,7),SV(1,1))
            DCSW = UNDOT3(PV(1,5),SV(1,4))
            DCSB = UNDOT3(PVJET(1,IBJ2),SV(1,3))
         ENDIF
         DEPL  = EVIS + EVOFF + ABS(PL) - ROOTS
         CSWW  = UNDOT3(PV(1,4),PV(1,5))
         CSBW1 = UNDOT3(PVJET(1,IBJ1),PV(1,4))
         CSBW2 = UNDOT3(PVJET(1,IBJ2),PV(1,5))
         CSBB  = UNDOT3(PVJET(1,IBJ1),PVJET(1,IBJ2))
C--
C  Cuts on b-b angle.
C--
         IF ( CSBB.GT.CSBBCT )                   GO TO 8000
C--
C  Cuts on b-W angles.
C--
         IF ( CSBW1.GT.CSBWCT .OR. CSBW2.GT.CSBWCT ) GO TO 8000
C--
C  Found an ttbar candidate.
C--
         NTT = NTT + 1
C>>>
CX       CHI2 =   ((AMW1-AMW)/SGMV)**2 + ((AMW2-AMW)/SGMV)**2
CX   .          + ((AMJ1-AMT)/SGMT)**2 + ((AMJ2-AMT)/SGMT)**2
         CHI2 =   ((AMW2-AMW)/SGMV)**2 + ((AMJ2-AMT)/SGMT)**2
C>>>
         IF ( CHI2.LT.CHI2S ) THEN
            CHI2S  = CHI2
            ITTBS2 = ITTBST
            ITTBST = NTT
            ICMBS2 = ICMBST
            ICMBST = ICMB
         ENDIF
C--
C  Save observables.
C--
         XHDATA( 1,NTT) = ICMB
         XHDATA( 2,NTT) = AMW1
         XHDATA( 3,NTT) = AMW2
         XHDATA( 5,NTT) = AMJ1
         XHDATA( 6,NTT) = AMJ2
         XHDATA( 7,NTT) = AMTT
         XHDATA(10,NTT) = CST1
         XHDATA(11,NTT) = CST2
         XHDATA(12,NTT) = EVIS
         XHDATA(13,NTT) = PT
         XHDATA(14,NTT) = PL
         XHDATA(15,NTT) = CHI2
         XHDATA(16,NTT) = DEPL
         XHDATA(17,NTT) = CTH1
         XHDATA(18,NTT) = CTH2
         XHDATA(19,NTT) = APT1
         XHDATA(20,NTT) = APT2
         XHDATA(21,NTT) = DCST
         XHDATA(22,NTT) = DCSW
         XHDATA(23,NTT) = DCSB
         XHDATA(24,NTT) = CSWW
         XHDATA(25,NTT) = CSBW1
         XHDATA(26,NTT) = CSBW2
         XHDATA(27,NTT) = CSBB
C--
         CALL HCUM1(24,AMTT,1.)
         CALL HCUM2(204,AMW1,AMW2,1.)
8000  CONTINUE
C--
C  Check if there is any combination left.
C--
      CALL HCUM1(25,NTT+.1,1.)
      IF ( ITTBST.LE.0 )                         GO TO 9999
      CALL HCUM1(1,11.1,1.)
C--
C  Thrust cut.
C  Thrust is calculated using all the tracks including missing
C  4-momentum.
C--
      CALL THRUST(NTRK+1,QV(1,1),LNxPV,0,THR,OBL,EIGVCT)
      CALL HCUM1(26,THR,1.)
      IF ( THR.GT.THRSCT )                       GO TO 9999
      CALL HCUM1(1,12.1,1.)
C--
C  Look at the best combination.
C--
      ITT = ITTBST
      AMW1  = XHDATA( 2,ITT)
      AMW2  = XHDATA( 3,ITT)
      AMT1  = XHDATA( 5,ITT)
      AMT2  = XHDATA( 6,ITT)
      AMTT  = XHDATA( 7,ITT)
      CST1  = XHDATA(10,ITT)
      CST2  = XHDATA(11,ITT)
      EVIS  = XHDATA(12,ITT)
      PT    = XHDATA(13,ITT)
      PL    = XHDATA(14,ITT)
      CHI2  = XHDATA(15,ITT)
      DEPL  = XHDATA(16,ITT)
      CTH1  = XHDATA(17,ITT)
      CTH2  = XHDATA(18,ITT)
      APT1  = XHDATA(19,ITT)
      APT2  = XHDATA(20,ITT)
      DCST  = XHDATA(21,ITT)
      DCSW  = XHDATA(22,ITT)
      DCSB  = XHDATA(23,ITT)
      CSWW  = XHDATA(24,ITT)
      CSBW1 = XHDATA(25,ITT)
      CSBW2 = XHDATA(26,ITT)
      CSBB  = XHDATA(27,ITT)
C--
      IF ( ITTBS2.GT.0 ) THEN
         AMTT2 = XHDATA( 7,ITTBS2)
         CHI22 = XHDATA(15,ITTBS2)
      ENDIF
C--
      CALL HCUM1(31,AMTT,1.)
      CALL HCUM1(32,CST1,1.)
      CALL HCUM1(33,CST2,1.)
      CALL HCUM1(34,PT  ,1.)
      CALL HCUM1(35,CHI2,1.)
      CALL HCUM1(36,YCUT(0),1.)
      CALL HCUM1(37,EOP,1.)
C--
      CALL HCUM2(221,AMT1,AMT2,1.)
      CALL HCUM2(223,EVIS,PL  ,1.)
      CALL HCUM2(229,AMW1 ,AMW2,1.)
      CALL HCUM2(231,ABS(CTH1),ABS(CTH2),1.)
      CALL HCUM2(233,ELP,ECON,1.)
      CALL HCUM2(234,CST2,DCST,1.)
      CALL HCUM2(235,DCSB,DCST,1.)
      CALL HCUM2(236,DCSW,DCST,1.)
      CALL HCUM2(237,CSWW,DCST,1.)
      CALL HCUM2(238,CSBW1,DCST,1.)
      CALL HCUM2(238,CSBW2,DCST,1.)
      CALL HCUM2(239,CSBB,DCST,1.)
      CALL HCUM2(240,AMW1,APT1,1.)
      CALL HCUM2(241,AMW1,DCST,1.)
      CALL HCUM2(242,AMW1,CST1,1.)
      CALL HCUM2(242,AMW1,CST2,1.)
      CALL HCUM2(243,CSTG,APTG,1.)
      CALL HCUM2(244,CST2,APT1,1.)
      CALL HCUM2(245,DCSB,APT1,1.)
      CALL HCUM2(246,DCSB,DCSW,1.)
      CALL HCUM2(247,AMW2,APT1,1.)
      CALL HCUM2(248,AMW2,DCST,1.)
      CALL HCUM2(249,APTG,APT1-APTG,1.)
      CALL HCUM2(250,AMT1,APT1,1.)
      CALL HCUM2(251,AMT2,APT1,1.)
      CALL HCUM2(252,APTG,DCST,1.)
      CALL HCUM2(253,ICMBGN+.1,ICMBST+.1,1.)
      CALL HCUM2(254,ICMBGN+.1,ICMBS2+.1,1.)
      CALL HCUM2(255,CSBB,APT1,1.)
      CALL HCUM2(256,AMT2,DCST,1.)
C--
      IF ( CSTG.LT.0 ) THEN
         IFBG = 0
      ELSE
         IFBG = 1
      ENDIF
      IF ( CST2.LT.0 ) THEN
         IFBR = 0
      ELSE
         IFBR = 1
      ENDIF
      IF ( IFBG.EQ.0 .AND. IFBR.EQ.0 ) THEN
         FB = 0.1
      ELSE IF ( IFBG.EQ.0 .AND. IFBR.EQ.1 ) THEN
         FB = 1.1
      ELSE IF ( IFBG.EQ.1 .AND. IFBR.EQ.1 ) THEN
         FB = 2.1
      ELSE
         FB = 3.1
      ENDIF
C--
      IF ( ICMBGN.EQ.ICMBST ) THEN
         CALL HCUM1( 42,NBJT+.1,1.)
         CALL HCUM1( 43,CSJJMX ,1.)
         CALL HCUM2(261,CSBB ,APT1 ,1.)
         CALL HCUM2(263,AMW1 ,AMW2 ,1.)
         CALL HCUM2(264,AMT1 ,AMT2 ,1.)
         CALL HCUM2(265,AMT2 ,APT1 ,1.)
         CALL HCUM2(266,AMT2 ,FB   ,1.)
      ELSE
         CALL HCUM1( 52,NBJT+.1,1.)
         CALL HCUM1( 53,CSJJMX ,1.)
         CALL HCUM2(271,CSBB ,APT1 ,1.)
         CALL HCUM2(273,AMW1 ,AMW2 ,1.)
         CALL HCUM2(274,AMT1 ,AMT2 ,1.)
         CALL HCUM2(275,AMT2 ,APT1 ,1.)
         CALL HCUM2(276,AMT2 ,FB   ,1.)
      ENDIF
C--
C  That's it.
C--
      IRET = 0
9999  RETURN
      END
