C***********************************************************************
C*
C*======================================----==
C* Subroutine PRUSER(IDREC,LEVEL,IDEBUG,IRET)
C*======================================----==
C*
C* (Function)
C*     User Event analysis routine.
C* (Input)
C*     IDREC   :  Record ID.
C*     ILEVEL  :  Level of analysis.
C*     IDEBUG  :  Debug flag.
C* (Output)
C*     IRET    :  Return flag.
C* (Update Record)
C*   95/06/14  K.Fujii		Original version for unix.
C*
C***********************************************************************
 
      SUBROUTINE PRUSER( IDREC, LEVEL, IDEBUG, IRET )
 
      CHARACTER*64    BKNAME(-3:3)
      DATA BKNAME  / 'Spring:Header',
     .               'Spring:Parton_List',
     .               'Generator:Header',
     .               'Generator:Particle_List' ,
     .               'Production:Combined_Lepton_Track',
     .               'Production:Combined_Gamma_Track',
     .               'Production:Combined_Hadron_Track' /
C--
      PARAMETER      ( LNxDT = 100, MXxPT = 5000, LNxPV = 10,
     .                 MXxPV = 500, MXxJET = 100, MXxYCT = 20 )
      COMMON /SSWORK/ IELM(MXxPT,-3:3), RDATA(LNxDT,MXxPT),
     .                PV(LNxPV,MXxPV), QV(LNxPV,MXxPV),SV(LNxPV,20),
     .                NJCLS(MXxYCT), NPINJ(MXxJET), IPTOJ(MXxPV),
     .                PVJET(4,MXxPV), XHDATA(30,500),
     .                EIGVCT(0:3,3), RBUF(LNxDT,MXxPT), INCN(0:MXxPT),
     .                SDATA(LNxDT,20), IPTDEX(MXxPV), IGNDEX(MXxPT),
     .                IBTAG(4)
      INTEGER*4       IDATA(LNxDT,MXxPT), NTK(0:3)
      EQUIVALENCE    (IDATA,RDATA)
      INTEGER*4       IUSFLG(MXxJET), IJTCMB(2,4,1000)
C--
C  Parameters for jet clustering.
C--
      REAL*4          YCUT(0:MXxYCT)
      DATA   YCUT     / 0.5E-4,
     .                  2.0E-4, 2.5E-4, 3.5E-4, 4.0E-4, 4.5E-4,
     .                  5.0E-4, 5.5E-4, 6.0E-4, 6.5E-4, 7.0E-4,
     .                  7.5E-4, 8.0E-4, 8.5E-4, 9.0E-4, 9.5E-4,
     .                  1.0E-3, 2.0E-3, 3.0E-3, 4.0E-3, 5.0E-3 /
C--
      DATA NOJ        /   8  /
      DATA NOMYCT     /   1  /
      DATA DYMAX      / 1.E-2/
      DATA NTRYMX     / 100  /
C--
C  Track selection.
C--
      DATA ECUT       /  0.1 /
C--
C  Energy and momentum balance.
C--
      DATA PTCUT      /  30. /
      DATA PLCUT      / 999. /
      DATA EVSCUT     / 500. /
      DATA EVOFF      / 10.0 /
      DATA DEPLCT     / -999./
C--
C  Mass cuts.
C--
C01      DATA AMJCUT     / 16.0 /
C01      DATA DMJCUT     / 25.0 /
      DATA AMJCUT     / 12.0 /
      DATA DMJCUT     / 25.0 /
C--
C  Parameters for best candidate selection.
C--
      DATA AMW        / 80.0 /
      DATA AMZ        / 91.2 /
      DATA AMT        / 170. /
C01      DATA SGMV       /   4. /
C01      DATA SGMT       /   8. /
      DATA SGMV       /   4. /
      DATA SGMT       /  15. /
C>>>
C
C==================<<  Entry Point  >>================================
C
C--
C  Reset return code, Skip except EVENT record.
C--
      IRET = 0
      IF( IDREC .NE. 1 ) RETURN
      LEVEL  = LEVEL
      IDEBUG = IDEBUG
C--
C  Cluster track matching.
C--
      CALL CMBTRK(IRT)
      IF ( IRT.LT.0 )                            GO TO 9999
C--
C  Set beam energy.
C--
      CALL TBGET(1,BKNAME(-1),1,NW,IDATA(1,1),IRT)
      IF ( IRT.LT.0 ) RETURN
      EBM   = IDATA(7,1)/1000.
      ROOTS = 2*EBM
C--
C  TBGET Spring information.
C        SDATA(*, 1) = t_bar
C             (*, 2) = t
C             (*, 3) = Z
C             (*, 4) = b_bar
C             (*, 5) = W-
C             (*, 6) = b
C             (*, 7) = W+
C             (*, 8) = f_U_bar
C             (*, 9) = f_D
C             (*,10) = f_U
C             (*,11) = f_D_bar
C             (*,12) = f_bar
C             (*,13) = f
C--
      NSP = 0
      IB  = -2
      CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
      DO 1000 IE = 1, NE
         NSP     = NSP + 1
         CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,SDATA(1,NSP),IRT)
         CALL UVCOPY(4,SDATA(5,NSP),SV(1,NSP))
1000  CONTINUE
      IGTT = NSP + 1
      CALL UADD4(SV(1,1),SV(1,2),SV(1,IGTT))
      AMTTG = UDOT4(SV(1,IGTT),SV(1,IGTT))
      AMTTG = SIGN(SQRT(ABS(AMTTG)),AMTTG)
      CALL HCUM1(27, AMTTG, 1.)
C--
C  For non-ttbar Z MC, set SDATA to dummy values.
C--
      IF ( NSP.NE.13 ) THEN
         DO 150 ISP = 1, 13
            SV(1,ISP) =  0.
            SV(2,ISP) =  0.
            SV(3,ISP) = 20.
            SV(4,ISP) = 20.
150      CONTINUE
      ENDIF
C--
C  TBGET simulator information.
C        RDATA(    1) = EX
C             (    2) = EY
C             (    3) = EZ
C             (    4) = E
C--
      NPT = 0
      NNU = 0
      CALL UVZERO(4,NTK)
      CALL UVZERO(MXxPT,IGNDEX)
      DO 2000 IB = 0, 3
         CALL TBNOEL(1,BKNAME(IB),NE,IELM(1,IB))
         IF ( NE.LE.0 )                          GO TO 2000
         DO 200 IE = 1, NE
            NPT     = NPT + 1
            NTK(IB) = NTK(IB) + 1
            CALL TBGET(1,BKNAME(IB),IELM(IE,IB),NW,RDATA(1,NPT),IRT)
            IF ( IB.EQ.0 ) THEN
               IGNDEX(IELM(IE,IB)) = NPT
               ID  = RDATA(2,NPT)
               IDA = IABS(ID)
               IF ( IDA.EQ.12 .OR. IDA.EQ.14 .OR. IDA.EQ.16 )
     .              NNU = NNU + 1
            ENDIF
200      CONTINUE
2000  CONTINUE
C--
C  # tracks.
C--
      CALL HCUM1(1,0.1,1.)
      CALL HCUM1(2,NPT+.1,1.)
      CALL HCUM1(3,NTK(1)+.1,1.)
      CALL HCUM1(4,NTK(2)+.1,1.)
      CALL HCUM1(5,NTK(3)+.1,1.)
C--
      IF ( NPT.EQ.0 )                            GO TO 9999
      CALL HCUM1(1,1.1,1.)
C--
C  Select good clusters.
C--
      IPT1 = NTK(0) + 1
      IPT2 = NPT
      NTRK = 0
      NLPT = 0
      DO 220  IPT = IPT1, IPT2
         E    = RDATA( 4,IPT)
         ITYP = RDATA(10,IPT)
         IF ( E.LT.ECUT )                        GO TO 220
         NTRK = NTRK + 1
         IF ( ITYP.EQ.11 .OR. ITYP.EQ.13 ) THEN
            IF ( E.GT.ELPLCT ) NLPT = NLPT + 1
         ENDIF
         CALL UVCOPY(4,RDATA(1,IPT),QV(1,NTRK))
         IPTDEX(NTRK) = IPT
220   CONTINUE
C--
      IF ( NTRK.LT.NOJ )                         GO TO 9999
      CALL HCUM1(1,2.1,1.)
C--
C  Require approximate 4-momentum conservation to select well
C  reconstructed events.
C--
      ISM = NTRK + 1
      ICM = NTRK + 2
      IMS = NTRK + 3
      CALL UVZERO(4,QV(1,ICM))
      CALL UVZERO(4,QV(1,ISM))
      QV(4,ICM) = ROOTS
      DO 250 ITK = 1, NTRK
         CALL UADD4(QV(1,ISM),QV(1,ITK),QV(1,ISM))
250   CONTINUE
      CALL USUB3(QV(1,ICM),QV(1,ISM),QV(1,IMS))
      EVIS = QV(4,ISM)
      PT   = SQRT( QV(1,ISM)*QV(1,ISM) + QV(2,ISM)*QV(2,ISM) )
      PL   = QV(3,ISM)
      CALL HCUM2(204,EVIS,PL,1.)
      CALL HCUM1(  8,PT,1.)
C--
      IF ( EVIS.LT.EVSCUT .OR. PT.GT.PTCUT .OR.
     .     ABS(PL).GT.PLCUT )                    GO TO 9999
      CALL HCUM1(1,3.1,1.)
C--
C  Cluster into jets.
C--
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,MXxYCT,YCUT(1),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
C--
      IF ( IRT.LT.0 )                            GO TO 9999
      CALL HCUM1(1,4.1,1.)
C--
      ICTS = 0
      DO 3000 ICT = 1, MXxYCT
         IF ( NJCLS(ICT).GE.NOJ ) ICTS = ICT
         DO 300 IJT = 2, 10
            IHST = 10 + IJT
            IF ( NJCLS(ICT).EQ.IJT ) THEN
               EPS = 1.
            ELSE IF ( NJCLS(ICT).GT.10 .AND. IJT.EQ.10 ) THEN
               EPS = 1.
            ELSE
               EPS = 0.
            ENDIF
            CALL HCUM1(IHST,ICT+.1,EPS)
300      CONTINUE
3000  CONTINUE
C--
C  Require 8 or more jets for nominal y_cut.
C--
      IF ( NJCLS(NOMYCT).LT.NOJ )                 GO TO 9999
      CALL HCUM1(1,5.1,1.)
C--
C  Find a y_cut which gives 8 jets.
C--
      NTRY    = 0
C--
      IF ( ICTS.LT.MXxYCT ) THEN
         YCUT(0) = YCUT(ICTS)
         DYCT    = (YCUT(ICTS+1)-YCUT(ICTS))/2
      ELSE IF ( NJCLS(ICTS).EQ.NOJ ) THEN
         YCUT(0) = YCUT(ICTS)
                                                 GO TO 360
      ELSE
         YCUT(0) = YCUT(MXxYCT)
         DYCT    = DYMAX
      ENDIF
350   NTRY    = NTRY + 1
      YCUT(0) = YCUT(0) + DYCT
      CALL JCLUST(NTRK,QV(1,1),LNxPV,0,1,YCUT(0),
     .            NJCLS,NPINJ,IPTOJ,PVJET,IRT)
      IF ( IRT.LT.0 ) THEN
         PRINT *, ' >>>>>>> Error in JCLUST >>>>>>>'
         PRINT *, '    IRT = ', IRT, ' Skip this event. '
                                                 GO TO 9999
      ENDIF
      NJT = NJCLS(1)
      IF ( NJT.EQ.NOJ )                          GO TO 360
      IF ( NTRY.GT.NTRYMX ) THEN
         IF ( NJT.EQ.NOJ )                       GO TO 360
         PRINT *, ' >>>>>> Cannot find y_cut >>>>>> '
         PRINT *, '    NTRY    = ', NTRY
         PRINT *, '    YCUT(0) = ', YCUT(0), ' NJT = ', NJT
         CALL HCUM1(9,NTRY+.1,1.)
                                                 GO TO 9999
      ENDIF
      IF ( NJT.LT.NOJ ) THEN
         DYCT = - ABS(DYCT)/2
      ELSE IF ( NJT.GE.NOJ ) THEN
         DYCT = ABS(DYCT)/2
      ENDIF
                                                 GO TO 350
360   CALL HCUM1(9,NTRY+.1,1.)
      CALL HCUM1(10,YCUT(0),1.)
C--
C  Look for W+, W-, and Z.
C--
      CALL UVZERO(MXxJET,IUSFLG)
      NCMB = 0
      DO 6000 IJT1 = 1, NOJ-1
         IF ( NPINJ(IJT1).EQ.1 )		 GO TO 6000
      DO 600 JJT1 = IJT1+1, NOJ
         IF ( NPINJ(JJT1).EQ.1 )		 GO TO 600
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         AMJ = UDOT4(PV(1,1),PV(1,1))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 600
         IUSFLG(IJT1) = 1
         IUSFLG(JJT1) = 1
      DO 5000 IJT2 = 1, NOJ-1
         IF ( IUSFLG(IJT2).NE.0 )                GO TO 5000
         IF ( NPINJ(IJT2).EQ.1 )		 GO TO 5000
      DO 500 JJT2 = IJT2+1, NOJ
         IF ( IUSFLG(JJT2).NE.0 )                GO TO 500
         IF ( NPINJ(JJT2).EQ.1 )		 GO TO 500
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         AMJ = UDOT4(PV(1,2),PV(1,2))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMW).GT.AMJCUT )           GO TO 500
         IUSFLG(IJT2) = 1
         IUSFLG(JJT2) = 1
      DO 4000 IJT3 = 1, NOJ-1
         IF ( IUSFLG(IJT3).NE.0 )                GO TO 4000
      DO 400 JJT3 = IJT3+1, NOJ
         IF ( IUSFLG(JJT3).NE.0 )                GO TO 400
         IF ( ( NPINJ(IJT3).EQ.1 .AND. NPINJ(JJT3).GT.1 ) .OR.
     .        ( NPINJ(IJT3).GT.1 .AND. NPINJ(JJT3).EQ.1 )) GO TO 400
         CALL UADD4(PVJET(1,IJT3),PVJET(1,JJT3),PV(1,3))
         AMJ = UDOT4(PV(1,3),PV(1,3))
         AMJ = SIGN(SQRT(ABS(AMJ)),AMJ)
         IF ( ABS(AMJ-AMZ).GT.AMJCUT )           GO TO 400
         IUSFLG(IJT3) = 1
         IUSFLG(JJT3) = 1
         NCMB = NCMB + 1
         IJTCMB(1,1,NCMB) = IJT1
         IJTCMB(2,1,NCMB) = JJT1
         IJTCMB(1,2,NCMB) = IJT2
         IJTCMB(2,2,NCMB) = JJT2
         IJTCMB(1,3,NCMB) = IJT3
         IJTCMB(2,3,NCMB) = JJT3
         NBJT = 0
         DO 40 IJT = 1, NOJ
            IF ( IUSFLG(IJT).EQ.0 ) THEN
               NBJT = NBJT + 1
               IJTCMB(NBJT,4,NCMB) = IJT
            ENDIF
40       CONTINUE
         IUSFLG(IJT3) = 0
         IUSFLG(JJT3) = 0
400   CONTINUE
4000  CONTINUE
         IUSFLG(IJT2) = 0
         IUSFLG(JJT2) = 0
500   CONTINUE
5000  CONTINUE
         IUSFLG(IJT1) = 0
         IUSFLG(JJT1) = 0
600   CONTINUE
6000  CONTINUE
C--
C  Calculate invariant masses, etc.
C--
      CALL HCUM1(6,NCMB+.1,1.)
      IF ( NCMB.LE.0 )                           GO TO 9999
      CALL HCUM1(1,6.1,1.)
C>>>
CP       PRINT *, ' --- '
CP       DO 660 IJT = 1, NOJ
CP          PRINT *, ' PVJET(*,', IJT, ') = ', ( PVJET(K,IJT), K=1,4 )
CP 660   CONTINUE
CP       PRINT *, ' PVJET_sum = ', ( QV(K,ISM), K=1,4 )
CP       PRINT *, ' --- '
CP       PRINT *, ' NCMB = ', NCMB
C>>>
      NTT    = 0
      CHI2S  = 1.E20
      ITTBST = 0
      ITTBS2 = 0
      DO 7000 ICMB = 1, NCMB
C>>>
CP          PRINT *, ' ICMB = ', ICMB, ' IJTCMB = ',
CP      .             ( ( IJTCMB(K,KK,ICMB), K=1,2 ), KK=1,4 )
C>>>
         IJT1 = IJTCMB(1,1,ICMB)
         JJT1 = IJTCMB(2,1,ICMB)
         IJT2 = IJTCMB(1,2,ICMB)
         JJT2 = IJTCMB(2,2,ICMB)
         IJT3 = IJTCMB(1,3,ICMB)
         JJT3 = IJTCMB(2,3,ICMB)
         IBJ1 = IJTCMB(1,4,ICMB)
         IBJ2 = IJTCMB(2,4,ICMB)
         CALL UADD4(PVJET(1,IJT1),PVJET(1,JJT1),PV(1,1))
         CALL UADD4(PVJET(1,IJT2),PVJET(1,JJT2),PV(1,2))
         CALL UADD4(PVJET(1,IJT3),PVJET(1,JJT3),PV(1,3))
         AMW1 = UDOT4(PV(1,1),PV(1,1))
         AMW1 = SIGN(SQRT(ABS(AMW1)),AMW1)
         AMW2 = UDOT4(PV(1,2),PV(1,2))
         AMW2 = SIGN(SQRT(ABS(AMW2)),AMW2)
         AMZ1 = UDOT4(PV(1,3),PV(1,3))
         AMZ1 = SIGN(SQRT(ABS(AMZ1)),AMZ1)
C>>>
C         CALL USETV4(AMW,PV(1,1),PV(1,4))
C         CALL USETV4(AMW,PV(1,2),PV(1,5))
         CALL UVCOPY(4,PV(1,1),PV(1,4))
         CALL UVCOPY(4,PV(1,2),PV(1,5))
C>>>
C--
         CALL UADD4(PV(1,4),PVJET(1,IBJ1),PV(1,6))
         CALL UADD4(PV(1,5),PVJET(1,IBJ2),PV(1,7))
         AMJ1 = UDOT4(PV(1,6),PV(1,6))
         AMJ1 = SIGN(SQRT(ABS(AMJ1)),AMJ1)
         AMJ2 = UDOT4(PV(1,7),PV(1,7))
         AMJ2 = SIGN(SQRT(ABS(AMJ2)),AMJ2)
C>>>
CP          PRINT *, ' AMT1, PV(*,T1) = ', AMJ1, ( PV(K,6), K=1,4 )
CP          PRINT *, ' AMT2, PV(*,T1) = ', AMJ2, ( PV(K,7), K=1,4 )
C>>>
         CALL HCUM2(201,AMJ1,AMJ2,1.)
         IF ( ABS(AMJ1-AMT).LT.DMJCUT .AND.
     .        ABS(AMJ2-AMT).LT.DMJCUT ) THEN
            NTT = NTT + 1
C--
            CHI2 =   ((AMW1-AMW)/SGMV)**2 + ((AMW2-AMW)/SGMV)**2
     .             + ((AMZ1-AMZ)/SGMV)**2
     .             + ((AMJ1-AMT)/SGMT)**2 + ((AMJ2-AMT)/SGMT)**2
            IF ( CHI2.LT.CHI2S ) THEN
               CHI2S  = CHI2
               ITTBS2 = ITTBST
               ITTBST = NTT
            ENDIF
C--
C>>>
C            CALL USETV4(AMT,PV(1,6),PV(1,8))
C            CALL USETV4(AMT,PV(1,7),PV(1,9))
            CALL UVCOPY(4,PV(1,6),PV(1,8))
            CALL UVCOPY(4,PV(1,7),PV(1,9))
C>>>
            CALL UADD4(PV(1,8),PV(1,9),PV(1,10))
            AMTT = UDOT4(PV(1,10),PV(1,10))
            AMTT = SIGN(SQRT(ABS(AMTT)),AMTT)
            CSZ  = PV(3,3)/UABSV3(PV(1,3))
            CST1 = PV(3,8)/UABSV3(PV(1,8))
            CST2 = PV(3,9)/UABSV3(PV(1,9))
            DEPL = EVIS + EVOFF + ABS(PL) - ROOTS
C-
            XHDATA( 1,NTT) = ICMB
            XHDATA( 2,NTT) = AMW1
            XHDATA( 3,NTT) = AMW2
            XHDATA( 4,NTT) = AMZ1
            XHDATA( 5,NTT) = AMJ1
            XHDATA( 6,NTT) = AMJ2
            XHDATA( 7,NTT) = AMTT
            XHDATA( 8,NTT) = PV(4,3)
            XHDATA( 9,NTT) = CSZ
            XHDATA(10,NTT) = CST1
            XHDATA(11,NTT) = CST2
            XHDATA(12,NTT) = EVIS
            XHDATA(13,NTT) = PT
            XHDATA(14,NTT) = PL
            XHDATA(15,NTT) = CHI2
            XHDATA(16,NTT) = DEPL
C--
            CALL HCUM1(7,AMTT,1.)
            CALL HCUM2(202,AMW1,AMW2,1.)
            CALL HCUM2(203,AMW1,AMZ1,1.)
         ENDIF
7000  CONTINUE
C--
C  Look at the best combination.
C--
      CALL HCUM1(11,NTT+.1,1.)
      IF ( ITTBST.LE.0 )                         GO TO 9999
      CALL HCUM1(1,7.1,1.)
C--
C  DEPL cut.
C--
      IF ( DEPL.LT.DEPLCT )			 GO TO 9999
      CALL HCUM1(1,8.1,1.)
C--
      ITT = ITTBST
      AMW1  = XHDATA( 2,ITT)
      AMW2  = XHDATA( 3,ITT)
      AMZ1  = XHDATA( 4,ITT)
      AMT1  = XHDATA( 5,ITT)
      AMT2  = XHDATA( 6,ITT)
      AMTT  = XHDATA( 7,ITT)
      EZ    = XHDATA( 8,ITT)
      CSZ   = XHDATA( 9,ITT)
      CST1  = XHDATA(10,ITT)
      CST2  = XHDATA(11,ITT)
      EVIS  = XHDATA(12,ITT)
      PT    = XHDATA(13,ITT)
      PL    = XHDATA(14,ITT)
      CHI2  = XHDATA(15,ITT)
      DEPL  = XHDATA(16,ITT)
      IF ( ITTBS2.GT.0 ) THEN
         AMTT2 = XHDATA( 7,ITTBS2)
         CHI22 = XHDATA(15,ITTBS2)
      ELSE
         AMTT2 = -999
         CHI22 = 9999
      ENDIF
C--
      CALL HCUM1(21,AMW1,1.)
      CALL HCUM1(21,AMW2,1.)
      CALL HCUM1(22,AMZ1,1.)
      CALL HCUM1(23,AMTT,1.)
      CALL HCUM1(24,CST1,1.)
      CALL HCUM1(24,CST2,1.)
      CALL HCUM1(25,PT  ,1.)
      CALL HCUM1(26,CHI2,1.)
      CALL HCUM2(221,AMT1,AMT2,1.)
      CALL HCUM2(222,CSZ ,EZ  ,1.)
      CALL HCUM2(223,EVIS,PL  ,1.)
      CALL HCUM2(224,CHI2,AMTT,1.)
      CALL HCUM2(225,PT  ,AMTT,1.)
      CALL HCUM2(226,CHI22,AMTT2,1.)
      CALL HCUM2(227,AMTT ,AMTT2,1.)
      CALL HCUM2(228,DEPL ,AMTT,1.)
      CALL HCUM2(229,AMW1 ,AMW2,1.)
      CALL HCUM2(230,AMW1 ,AMZ1,1.)
      CALL HCUM2(231,AMTT ,AMTTG,1.)
C--
C  That's it.
C--
9999  RETURN
      END
