      SUBROUTINE J3XXXP(FI,FO,GAF,GZF,ZMASS,ZWIDTH , J3)
C
C This subroutine computes the sum of photon and Z currents with the    
C suitable weights ( J(W3) = cos(theta_W) J(Z) + sin(theta_W) J(A) ).   
C The output J3 is useful as an input of VVVXXX, JVVXXX or W3W3XX.      
C The photon propagator is given in Feynman gauge, and the Z propagator 
C is given in unitary gauge.                                            
C                                                                       
C INPUT:                                                                
C       complex FI(6)          : flow-in  fermion                   |FI>
C       complex FO(6)          : flow-out fermion                   <FO|
C       real    GAF(2)         : FI couplings with A                 GAF
C       real    GZF(2)         : FI couplings with Z                 GZF
C       real    ZMASS          : mass  of Z                             
C       real    ZWIDTH         : width of Z                             
C                                                                       
C OUTPUT:                                                               
C       complex J3(6)          : W3 current             J^mu(<FO|W3|FI>)
C
      IMPLICIT    REAL*8 ( A-H, O-Z )
      COMPLEX*16  FI(6),FO(6),J3(6), WRK(6,2)
      REAL*8      GAF(2),GZF(2),ZMASS,ZWIDTH
C
      CW=1/SQRT(1+(GZF(2)/GAF(2))**2)
      SW=SQRT((1-CW)*(1+CW))
      AMASS  = 0
      AWIDTH = 0
C
      CALL JIOXXX(FI,FO,GAF,AMASS,AWIDTH,WRK(1,1))
      CALL JIOXXX(FI,FO,GZF,ZMASS,ZWIDTH,WRK(1,2))
      J3(1) = SW*WRK(1,1) + CW*WRK(1,2)
      J3(2) = SW*WRK(2,1) + CW*WRK(2,2)
      J3(3) = SW*WRK(3,1) + CW*WRK(3,2)
      J3(4) = SW*WRK(4,1) + CW*WRK(4,2)
      J3(5) = WRK(5,1)
      J3(6) = WRK(6,1)
C
      RETURN
      END
