CC**********************************************************************
C* 
C*==============================-----===
C* Subroutine SGEEWWF(IDP,IHEL,X,SG,PV)
C*==============================-----===
C*
C* (Purpose)
C*    Calculate differential cross section for e+ + e- ---> e+ e- W+ W-.
C*    with the Z's subsequent decays.
C* (Inputs)
C*       IDP(1,i) : (I*4) : generation number.
C*          (2,i) : (I*4) : T3 + 1.5.
C*          (3,i) : (I*4) : l/q flag; (1,2) = (l,q).
C*       IHEL( i) : (I*4) : helicity combination.
C*                        : i = (1,2,3,4,5,6)
C*                        :   = (e-,e+, e-,e+, W-,W+)
C*       X(1,1)   : (R*8) : s.
C*        (2,1)   : (R*8) : eta_1.
C*        (3,1)   : (R*8) : phi_1.
C*       X(1,2)   : (R*8) : log(1-E_i/E_bm).
C*                        : ( 1/(1-E_i/E_bm) ).
C*        (2,2)   : (R*8) : eta_2.
C*        (3,2)   : (R*8) : phi_2.
C*       X(1,3)   : (R*8) : invariant mass squared for W-W+.
C*        (2,3)   : (R*8) : cos_W- in (W-W+) rest frame.
C*        (3,3)   : (R*8) : phi_W- in (W-W+) rest frame.
C* (Output)
C*       PV(*,i)  : (R*8) : 4-momentum of i-th particle, where
C*                        : numbering convention is that of IDP.
C*       SG       : (R*8) : d(sigma)/dX1dX2....dXn.
C* (Relation)
C*    Invokes UVZERO, UHSETF, UBTRAN, FULEEWW, and routines
C*    in the HELAS library.
C* (Update Record)
C*    95/04/26  K.Fujii		Original version.
C*
CC**********************************************************************
 
      SUBROUTINE SGEEWWF(IDP,IHEL,X,SG,PV)
 
      IMPLICIT     REAL*8  ( A-H, O-Z )
=EXPAND 'dkin_lib/SMCONS.inc'
=EXPAND 'dkin_lib/SMPTAB.inc'
      PARAMETER    ( NP = 6, NB = NP - 3 )
      INTEGER*4    IDP(3,NP), IHEL(NP)
      REAL   *8    SG, PV(0:3,NP)
      REAL   *8    X(3,NB)
C--
      REAL   *8    QV(0:3,NP), EC(3,3), QT(0:3), QX(0:3), QZ(0:3),
     .             XX(3,2), AMP2(0:3)
      REAL   *8    QV32
      DATA NCALL /0/
C--
C  Statement function.
C--
      BETA2(X1,X2) =  1 - 2*(X1+X2) + (X1-X2)**2
C
C========< Entry Point >================================================
C
C--
C  Constants.
C  Spin average is taken only for positron.
C--
      IF ( NCALL.EQ.0 ) THEN
         NCALL  = 1
         SPIN   = 2
         FACT   = xGV2PB*x2PI/(x4PI**(3*NB))/SPIN
         FACT   = FACT/(xALF*137.0359895D0)**2
         AME    = AMF(1,2,1)
         AMSW   = AMW
         CALL UVZERO(18,EC)
         EC(1,1) = 1
         EC(2,2) = 1
         EC(3,3) = 1
      ENDIF
C--
C  Reset phase space weight.
C--
      WAT = 1
C--
C  Define some kinematical variables.
C--
      S       = X(1,1)
      RS      = SQRT(S)
      EBM     = RS/2
C--
      AM3     = AMF(IDP(1,3),IDP(2,3),IDP(3,3))
      AM4     = AMF(IDP(1,4),IDP(2,4),IDP(3,4))
      Q3      = SQRT(X(1,3))
      Q4      = AMSW
      Q5      = AMSW
C--
      RXI     = EXP(X(1,2))
      WAT     = WAT*RXI
C--
      DM1     = (AME*AME/S)*RXI*RXI/(1-RXI)
      DP1     = (AME*AME/S)
      DM2     = DP1
      DP2     = DM1
C--
      ET1     = X(2,1)
      EXP2ET  = EXP(2*ET1)
      SH1     = MIN( SQRT( (1+DP1+DM1)/(1+EXP2ET)   - DM1 ), 1.D0 )
      CH1     = MIN( SQRT( (1+DP1+DM1)/(1+1/EXP2ET) - DP1 ), 1.D0 )
      SN1     = 2*SH1*CH1
      CS1     = (1+DP1+DM1)*TANH(ET1) - DP1 + DM1
      FI1     = X(3,1)
      WAT     = WAT*4*(1+DP1+DM1)/((1+EXP2ET)*(1+1/EXP2ET))
C--
      ET2     = X(2,2)
      EXP2ET  = EXP(2*ET2)
      SH2     = MIN( SQRT( (1+DP2+DM2)/(1+EXP2ET)   - DM2 ), 1.D0 )
      CH2     = MIN( SQRT( (1+DP2+DM2)/(1+1/EXP2ET) - DP2 ), 1.D0 )
      SN2     = 2*SH2*CH2
      CS2     = (1+DP2+DM2)*TANH(ET2) - DP2 + DM2
      FI2     = X(3,2)
      WAT     = WAT*4*(1+DP2+DM2)/((1+EXP2ET)*(1+1/EXP2ET))
C--
      CS12    = CS1*CS2 + SN1*SN2*COS(FI2-FI1)
C--
      IF ( ET1.GT.-ET2 ) THEN
         RXIMN   = (Q3-AM3+AM4)*(Q3+AM3+AM4)/S
         RXIMX   = 1 - 2*AM3/RS
         X1      = 1 - RXI
         X2      = ( RXI - X(1,3)/S )/( 1 - X1*(1-CS12)/2 )
         IF ( 1-X2.LT.RXIMN .OR. 1-X2.GT.RXIMX )	GO TO 9999
         WAT     = WAT*S*X1*X2*X2/( RXI - X(1,3)/S )
      ELSE
         RXIMN   = (Q3+AM3-AM4)*(Q3+AM3+AM4)/S
         RXIMX   = 1 - 2*AM4/RS
         X2      = 1 - RXI
         X1      = ( RXI - X(1,3)/S )/( 1 - X2*(1-CS12)/2 )
         IF ( 1-X1.LT.RXIMN .OR. 1-X1.GT.RXIMX )	GO TO 9999
         WAT     = WAT*S*X2*X1*X1/( RXI - X(1,3)/S )
      ENDIF
C--
C  Set 4-momenta.
C--
      CALL UVZERO(8*NP,PV)
      CALL UVZERO(8*NP,QV)
C
C-- e+ + e- --> CM.
C
      PBM     = SQRT((EBM-AME)*(EBM+AME))
      BT0     = PBM/EBM
      PV(0,1) = EBM
      PV(3,1) = PBM
      CALL PMIRRx(PV(0,1),PV(0,2))
      QV(0,1) = RS
C
C-- CM --> a + b + X.
C
C-(a)
      E1      = EBM*X1
      IF ( E1.LT.AM3 )				 GO TO 9999
      APV     = SQRT((E1-AM3)*(E1+AM3))
      PV(0,3) = E1
      PV(1,3) = APV*SN1*COS(FI1)
      PV(2,3) = APV*SN1*SIN(FI1)
      PV(3,3) = APV*CS1
C-(b)
      E2      = EBM*X2
      IF ( E2.LT.AM4 ) 				 GO TO 9999
      APV     = SQRT((E2-AM4)*(E2+AM4))
      PV(0,4) = E2
      PV(1,4) = APV*SN2*COS(FI2)
      PV(2,4) = APV*SN2*SIN(FI2)
      PV(3,4) = APV*CS2
C-(X)
      CALL PSUBxx(QV(0,1),PV(0,3),QV(0,3))
      CALL PSUBxx(QV(0,3),PV(0,4),QV(0,3))
C
C-- X --> C + D.
C
      CALL RDOTxx(QV(0,3),QV(0,3),QV32)
      AM12 = Q4**2
      AM22 = Q5**2
      BT2  = BETA2(AM12/QV32,AM22/QV32)
      IF ( QV32.LT.0.D0 .OR. BT2.LE.0.D0 )	 GO TO 9999
C--
      BT2  = SQRT(BT2)
      QV3  = SQRT(QV32)
      APV  = (QV3/2)*BT2
      COS1 = X(2,3)
      PHI1 = X(3,3)
      SIN1 = SQRT((1-COS1)*(1+COS1))
      PV(0,5) = SQRT(APV*APV+AM12)
      PV(1,5) = APV*SIN1*COS(PHI1)
      PV(2,5) = APV*SIN1*SIN(PHI1)
      PV(3,5) = APV*COS1
      PV(0,6) = QV3 - PV(0,5)
      PV(1,6) =     - PV(1,5)
      PV(2,6) =     - PV(2,5)
      PV(3,6) =     - PV(3,5)
C--
      CALL PSUBxx(PV(0,1),PV(0,3),QT)
      CALL PMIRRx(QV(0,3),QX)
      CALL BOOSTx(QT,QX,QZ)
      CALL ROTxxx(PV(0,5),QZ,QV(0,5))
      CALL ROTxxx(PV(0,6),QZ,QV(0,6))
      CALL BOOSTx(QV(0,5),QV(0,3),PV(0,5))
      CALL BOOSTx(QV(0,6),QV(0,3),PV(0,6))
C--
      XX(1,1) = SH1
      XX(2,1) = CH1
      XX(3,1) = FI1
      XX(1,2) = SH2
      XX(2,2) = CH2
      XX(3,2) = FI2
C--
C  Calculate phase space weight.
C--
      WAT = WAT*FACT*BT2/(2*S*BT0)
C--
C  Calculate amplitude squared.
C--
      CALL FULEEWW(IDP,IHEL,XX,PV,AMP2)
C--
C  Differenctial cross section.
C--
      SG  = AMP2(0)*WAT
C--
C  That's it.
C--
      RETURN
C--
C  Kinematically forbidden.
C--
9999  SG = 0
      RETURN
      END
